@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/mediafilespreviewer.py
-#
 # Copyright (c) 2011, Pier Carteri <pier.carteri@gmail.com>
 # Copyright (c) 2012, Thibault Saunier <tsaunier@gnome.org>
 #
@@ -20,42 +17,45 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
+import html
 from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
+from gi.repository import GES
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gst
 from gi.repository import Gtk
-from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import Pango
-from gi.repository import GstPbutils
 
 from pitivi.settings import GlobalSettings
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import uri_is_valid
 from pitivi.utils.pipeline import AssetPipeline
-from pitivi.utils.ui import beautify_length, beautify_stream, SPACING
-from pitivi.viewer import ViewerWidget
+from pitivi.utils.ui import beautify_length
+from pitivi.utils.ui import beautify_stream
+from pitivi.utils.ui import SPACING
+from pitivi.viewer.viewer import ViewerWidget
 
 PREVIEW_WIDTH = 250
 PREVIEW_HEIGHT = 100
 
 GlobalSettings.addConfigSection('filechooser-preview')
 GlobalSettings.addConfigOption('FCEnablePreview',
-    section='filechooser-preview',
-    key='do-preview-on-clip-import',
-    default=True)
+                               section='filechooser-preview',
+                               key='do-preview-on-clip-import',
+                               default=True)
 GlobalSettings.addConfigOption('FCpreviewWidth',
-    section='filechooser-preview',
-    key='video-preview-width',
-    default=PREVIEW_WIDTH)
+                               section='filechooser-preview',
+                               key='video-preview-width',
+                               default=PREVIEW_WIDTH)
 GlobalSettings.addConfigOption('FCpreviewHeight',
-    section='filechooser-preview',
-    key='video-preview-height',
-    default=PREVIEW_HEIGHT)
+                               section='filechooser-preview',
+                               key='video-preview-height',
+                               default=PREVIEW_HEIGHT)
 
-acceptable_tags = [
+ACCEPTABLE_TAGS = [
     Gst.TAG_ALBUM_ARTIST,
     Gst.TAG_ARTIST,
     Gst.TAG_TITLE,
@@ -69,11 +69,10 @@ acceptable_tags = [
 
 
 class PreviewWidget(Gtk.Grid, Loggable):
-    """
-    Widget for displaying a GStreamer sink with playback controls.
+    """Widget for displaying a GStreamer sink with playback controls.
 
-    @ivar settings: The settings of the app.
-    @type settings: L{GlobalSettings}
+    Args:
+        settings (GlobalSettings): The settings of the app.
     """
 
     def __init__(self, settings, minimal=False):
@@ -81,13 +80,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
         Loggable.__init__(self)
 
         self.log("Init PreviewWidget")
-        self.connect('destroy', self._destroy_cb)
-
         self.settings = settings
-        self.preview_cache = {}
-        self.preview_cache_errors = {}
-
-        self.discoverer = GstPbutils.Discoverer.new(Gst.SECOND)
+        self.error_message = None
 
         # playbin for play pics
         self.player = AssetPipeline(clip=None, name="preview-player")
@@ -97,31 +91,33 @@ class PreviewWidget(Gtk.Grid, Loggable):
 
         # some global variables for preview handling
         self.is_playing = False
+        self.at_eos = False
         self.original_dims = (PREVIEW_WIDTH, PREVIEW_HEIGHT)
         self.countinuous_seek = False
         self.slider_being_used = False
         self.current_selected_uri = ""
         self.current_preview_type = ""
+        self.play_on_discover = False
         self.description = ""
-        self.tags = {}
 
         # Gui elements:
         # Drawing area for video output
-        self.preview_video = ViewerWidget(
-            realizedCb=self._on_preview_video_realize_cb)
-        self.preview_video.sink = self.player.video_sink
+        self.preview_video = ViewerWidget(self.player.sink_widget)
         self.preview_video.props.hexpand = minimal
         self.preview_video.props.vexpand = minimal
+        self.preview_video.show_all()
         self.attach(self.preview_video, 0, 0, 1, 1)
 
         # An image for images and audio
         self.preview_image = Gtk.Image()
-        self.preview_image.set_size_request(self.settings.FCpreviewWidth, self.settings.FCpreviewHeight)
+        self.preview_image.set_size_request(
+            self.settings.FCpreviewWidth, self.settings.FCpreviewHeight)
         self.preview_image.show()
         self.attach(self.preview_image, 0, 1, 1, 1)
 
         # Play button
-        self.bbox = Gtk.HBox()
+        self.bbox = Gtk.Box()
+        self.bbox.set_orientation(Gtk.Orientation.HORIZONTAL)
         self.play_button = Gtk.ToolButton()
         self.play_button.set_icon_name("media-playback-start")
         self.play_button.connect("clicked", self._on_start_stop_clicked_cb)
@@ -144,8 +140,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
         self.b_zoom_out = Gtk.ToolButton()
         self.b_zoom_out.set_icon_name("zoom-out")
         self.b_zoom_out.connect("clicked", self._on_zoom_clicked_cb, -1)
-        self.bbox.pack_start(self.b_zoom_in, False, True, 0)
-        self.bbox.pack_start(self.b_zoom_out, False, True, 0)
+        self.bbox.pack_start(self.b_zoom_in, False, False, 0)
+        self.bbox.pack_start(self.b_zoom_out, False, False, 0)
         self.bbox.show_all()
         self.attach(self.bbox, 0, 2, 1, 1)
 
@@ -157,7 +153,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
         self.attach(self.l_tags, 0, 3, 1, 1)
 
         # Error handling
-        vbox = Gtk.VBox()
+        vbox = Gtk.Box()
+        vbox.set_orientation(Gtk.Orientation.VERTICAL)
         vbox.set_spacing(SPACING)
         self.l_error = Gtk.Label(label=_("Pitivi can not preview this file."))
         self.b_details = Gtk.Button.new_with_label(_("More info"))
@@ -172,48 +169,45 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.bbox.remove(self.b_zoom_in)
             self.bbox.remove(self.b_zoom_out)
 
-    def add_preview_request(self, dialogbox):
-        """add a preview request """
-        uri = dialogbox.get_preview_uri()
-        if uri is None or not uri_is_valid(uri):
-            return
-        self.previewUri(uri)
+        self.clear_preview()
 
-    def previewUri(self, uri):
+    def update_preview_cb(self, file_chooser):
+        """Previews the URI of the specified file chooser.
+
+        Args:
+            file_chooser (Gtk.FileChooser): The file chooser providing the URI.
+        """
+        uri = file_chooser.get_preview_uri()
+        previewable = uri and uri_is_valid(uri)
+        if not previewable:
+            self.clear_preview()
+            return
+        self.preview_uri(uri)
+
+    def preview_uri(self, uri):
         self.log("Preview request for %s", uri)
         self.clear_preview()
         self.current_selected_uri = uri
-        if uri in self.preview_cache:  # Already discovered
-            self.log(uri + " already in cache")
-            self.show_preview(uri, None)
-        elif uri in self.preview_cache_errors:
-            self.log(uri + " already in error cache")
-            self.show_error(uri)
-        else:
-            self.log("Call discoverer for " + uri)
-            self.fixme("Use a GESAsset here, and discover async with it")
-            try:
-                info = self.discoverer.discover_uri(uri)
-            except Exception as e:
-                self.preview_cache_errors[uri] = e
-                if self.current_selected_uri == uri:
-                    self.show_error(uri)
-                return
+        GES.UriClipAsset.new(uri, None, self.__asset_loaded_cb)
 
-            if self.current_selected_uri == uri:
-                self.show_preview(uri, info)
-
-    def show_preview(self, uri, info):
-        if info:
-            self.preview_cache[uri] = info
-        else:
-            self.log("Show preview for " + uri)
-            info = self.preview_cache.get(uri, None)
-
-        if info is None:
-            self.log("No preview for " + uri)
+    def __asset_loaded_cb(self, source, res):
+        uri = source.get_id()
+        try:
+            asset = GES.Asset.request_finish(res)
+        except GLib.Error as error:
+            self.log("Failed discovering %s: %s", uri, error.message)
+            self._show_error(error.message)
             return
 
+        self.log("Discovered %s", uri)
+        if not self._show_preview(uri, asset.get_info()):
+            return
+        if self.play_on_discover:
+            self.play_on_discover = False
+            self.play()
+
+    def _show_preview(self, uri, info):
+        self.log("Show preview for %s", uri)
         duration = info.get_duration()
         pretty_duration = beautify_length(duration)
 
@@ -223,13 +217,21 @@ class PreviewWidget(Gtk.Grid, Loggable):
             if video.is_image():
                 self.current_preview_type = 'image'
                 self.preview_video.hide()
-                pixbuf = GdkPixbuf.Pixbuf.new_from_file(Gst.uri_get_location(uri))
+                path = Gst.uri_get_location(uri)
+                try:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
+                except GLib.Error as error:
+                    self.debug("Failed loading image because: %s", error)
+                    self._show_error(error.message)
+                    return False
                 pixbuf_w = pixbuf.get_width()
                 pixbuf_h = pixbuf.get_height()
                 w, h = self.__get_best_size(pixbuf_w, pixbuf_h)
-                pixbuf = pixbuf.scale_simple(w, h, GdkPixbuf.InterpType.NEAREST)
+                pixbuf = pixbuf.scale_simple(
+                    w, h, GdkPixbuf.InterpType.NEAREST)
                 self.preview_image.set_from_pixbuf(pixbuf)
-                self.preview_image.set_size_request(self.settings.FCpreviewWidth, self.settings.FCpreviewHeight)
+                self.preview_image.set_size_request(
+                    self.settings.FCpreviewWidth, self.settings.FCpreviewHeight)
                 self.preview_image.show()
                 self.bbox.show()
                 self.play_button.hide()
@@ -242,11 +244,13 @@ class PreviewWidget(Gtk.Grid, Loggable):
                 self.player.setClipUri(self.current_selected_uri)
                 self.player.setState(Gst.State.PAUSED)
                 self.pos_adj.props.upper = duration
-                video_width = (video.get_par_num() / video.get_par_denom()) * video.get_width()
+                video_width = (
+                    video.get_par_num() / video.get_par_denom()) * video.get_width()
                 video_height = video.get_height()
                 w, h = self.__get_best_size(video_width, video_height)
                 self.preview_video.set_size_request(w, h)
-                self.preview_video.setDisplayAspectRatio(float(video_width) / video_height)
+                aspect_ratio = video_width / video_height
+                self.preview_video.setDisplayAspectRatio(aspect_ratio)
                 self.preview_video.show()
                 self.bbox.show()
                 self.play_button.show()
@@ -254,19 +258,21 @@ class PreviewWidget(Gtk.Grid, Loggable):
                 self.b_zoom_in.show()
                 self.b_zoom_out.show()
                 self.description = "\n".join([
-                    _("<b>Resolution</b>: %d×%d") % (video_width, video_height),
+                    _("<b>Resolution</b>: %d×%d") % (
+                        video_width, video_height),
                     _("<b>Duration</b>: %s") % pretty_duration])
         else:
             self.current_preview_type = 'audio'
             self.preview_video.hide()
             audio = info.get_audio_streams()
-
             if not audio:
-                return
+                self.debug("Audio has no streams")
+                return False
 
             audio = audio[0]
             self.pos_adj.props.upper = duration
-            self.preview_image.set_from_icon_name("audio-x-generic", Gtk.IconSize.DIALOG)
+            self.preview_image.set_from_icon_name(
+                "audio-x-generic", Gtk.IconSize.DIALOG)
             self.preview_image.show()
             self.preview_image.set_size_request(PREVIEW_WIDTH, PREVIEW_HEIGHT)
             self.description = "\n".join([
@@ -280,20 +286,30 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.b_zoom_in.hide()
             self.b_zoom_out.hide()
             self.bbox.show()
+        return True
 
-    def show_error(self, unused_uri):
+    def _show_error(self, error_message):
+        self.error_message = error_message
         self.l_error.show()
         self.b_details.show()
 
     def play(self):
+        if not self.current_preview_type:
+            self.play_on_discover = True
+            return
+        if self.at_eos:
+            # The content played once already and the pipeline is at the end.
+            self.at_eos = False
+            self.player.simple_seek(0)
         self.player.setState(Gst.State.PLAYING)
         self.is_playing = True
         self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PAUSE)
         GLib.timeout_add(250, self._update_position)
         self.debug("Preview started")
 
-    def pause(self):
-        self.player.setState(Gst.State.PAUSED)
+    def pause(self, state=Gst.State.PAUSED):
+        if state is not None:
+            self.player.setState(state)
         self.is_playing = False
         self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
         self.log("Preview paused")
@@ -312,10 +328,7 @@ class PreviewWidget(Gtk.Grid, Loggable):
         self.b_details.hide()
         self.description = ""
         self.l_tags.set_markup("")
-        self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
-        self.player.setState(Gst.State.NULL)
-        self.is_playing = False
-        self.tags = {}
+        self.pause(state=Gst.State.NULL)
         self.current_selected_uri = ""
         self.current_preview_type = ""
         self.preview_image.hide()
@@ -331,6 +344,7 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.countinuous_seek = False
             value = int(widget.get_value())
             self.player.simple_seek(value)
+            self.at_eos = False
             if self.is_playing:
                 self.player.setState(Gst.State.PLAYING)
             # Now, allow gobject timeout to continue updating the slider pos:
@@ -340,16 +354,17 @@ class PreviewWidget(Gtk.Grid, Loggable):
         if self.countinuous_seek:
             value = int(widget.get_value())
             self.player.simple_seek(value)
+            self.at_eos = False
 
     def _pipelineEosCb(self, unused_pipeline):
-        self.player.setState(Gst.State.NULL)
-        self.is_playing = False
-        self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PLAY)
-        self.pos_adj.set_value(0)
+        self._update_position()
+        self.pause()
+        # The pipeline is at the end. Leave it like that so the last frame
+        # is displayed.
+        self.at_eos = True
 
-    def _pipelineErrorCb(self, unused_message, unused_detail):
-        self.player.setState(Gst.State.NULL)
-        self.is_playing = False
+    def _pipelineErrorCb(self, unused_pipeline, unused_message, unused_detail):
+        self.pause(state=Gst.State.NULL)
 
     def _update_position(self, *unused_args):
         if self.is_playing and not self.slider_being_used:
@@ -357,15 +372,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.pos_adj.set_value(int(curr_pos))
         return self.is_playing
 
-    def _on_preview_video_realize_cb(self, unused_drawing_area, unused_widget):
-        if self.current_preview_type == 'video':
-            self.player.connectWithViewer(self.preview_video)
-
     def _on_start_stop_clicked_cb(self, button):
-        if self.is_playing:
-            self.pause()
-        else:
-            self.play()
+        self.togglePlayback()
 
     def _on_zoom_clicked_cb(self, button, increment):
         if self.current_preview_type == 'video':
@@ -393,8 +401,10 @@ class PreviewWidget(Gtk.Grid, Loggable):
                 h *= 0.8
                 if (w, h) < self.original_dims:
                     (w, h) = self.original_dims
-            pixbuf = GdkPixbuf.Pixbuf.new_from_file(Gst.uri_get_location(self.current_selected_uri))
-            pixbuf = pixbuf.scale_simple(int(w), int(h), GdkPixbuf.InterpType.BILINEAR)
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file(
+                Gst.uri_get_location(self.current_selected_uri))
+            pixbuf = pixbuf.scale_simple(
+                int(w), int(h), GdkPixbuf.InterpType.BILINEAR)
 
             w = max(w, self.settings.FCpreviewWidth)
             h = max(h, self.settings.FCpreviewHeight)
@@ -404,8 +414,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.settings.FCpreviewWidth = int(w)
             self.settings.FCpreviewHeight = int(h)
 
-    def _appendTag(self, taglist, tag, unused_udata):
-        if tag in acceptable_tags:
+    def _append_tag(self, taglist, tag, tags):
+        if tag in ACCEPTABLE_TAGS:
             name = Gst.tag_get_nick(tag)
             type = Gst.tag_get_type(tag)
             type_getters = {GObject.TYPE_STRING: 'get_string',
@@ -414,38 +424,42 @@ class PreviewWidget(Gtk.Grid, Loggable):
                             GObject.TYPE_INT: 'get_int',
                             GObject.TYPE_UINT: 'get_uint'}
             if type in type_getters:
-                if type == GObject.TYPE_STRING:
-                    value = getattr(taglist, type_getters[type])(tag)[1]
-                    value = value.replace('<', ' ').replace('>', ' ')
-                else:
-                    value = str(getattr(taglist, type_getters[type])(tag)[1])
-                self.tags[name] = value
+                res, value = getattr(taglist, type_getters[type])(tag)
+                assert res
+                if not type == GObject.TYPE_STRING:
+                    value = str(value)
+                tags[name] = value
 
-    def _tag_found_cb(self, abus, mess):
-        tag_list = mess.parse_tag()
-        tag_list.foreach(self._appendTag, None)
-        keys = list(self.tags.keys())
-        keys.sort()
+    def _tag_found_cb(self, unused_bus, message):
+        tag_list = message.parse_tag()
+        tags = {}
+        tag_list.foreach(self._append_tag, tags)
+        items = list(tags.items())
+        items.sort()
         text = self.description + "\n\n"
-        for key in keys:
-            text = text + "<b>" + key.capitalize() + "</b>: " + self.tags[key] + "\n"
+        for key, value in items:
+            capitalized = key.capitalize()
+            escaped = html.escape(value)
+            text = text + "<b>%s</b>: %s\n" % (key, escaped)
         self.l_tags.set_markup(text)
 
     def _on_b_details_clicked_cb(self, unused_button):
-        mess = self.preview_cache_errors.get(self.current_selected_uri, None)
-        if mess is not None:
-            dialog = Gtk.MessageDialog(transient_for=None,
-                modal=True,
-                message_type=Gtk.MessageType.WARNING,
-                buttons=Gtk.ButtonsType.OK,
-                text=str(mess))
-            dialog.set_icon_name("pitivi")
-            dialog.set_title(_("Error while analyzing a file"))
-            dialog.run()
-            dialog.destroy()
+        if not self.error_message:
+            return
 
-    def _destroy_cb(self, widget):
-        self.player.setState(Gst.State.NULL)
+        dialog = Gtk.MessageDialog(transient_for=None,
+                                   modal=True,
+                                   message_type=Gtk.MessageType.WARNING,
+                                   buttons=Gtk.ButtonsType.OK,
+                                   text=self.error_message)
+        dialog.set_icon_name("pitivi")
+        dialog.set_title(_("Error while analyzing a file"))
+        dialog.run()
+        dialog.destroy()
+
+    def do_destroy(self):
+        """Handles the destruction of the widget."""
+        self.player.release()
         self.is_playing = False
 
     def __get_best_size(self, width_in, height_in):
