@@ -18,11 +18,11 @@
 # Boston, MA 02110-1301, USA.
 """Rendering-related classes and utilities."""
 import os
-import subprocess
 import time
 from gettext import gettext as _
 
 from gi.repository import GES
+from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gst
@@ -115,8 +115,6 @@ class Encoders(Loggable):
             # We have to initialize the instance here, otherwise
             # __init__ is called every time we use Encoders().
             Loggable.__init__(cls._instance)
-            Gst.Registry.get().connect(
-                "feature-added", cls._instance._registry_feature_added_cb)
             cls._instance._load_encoders()
             cls._instance._load_combinations()
         return cls._instance
@@ -220,10 +218,6 @@ class Encoders(Loggable):
             factory in self.supported_aencoders or\
             factory in self.supported_vencoders
 
-    def _registry_feature_added_cb(self, registry, feature):
-        # TODO Check what feature has been added and update our lists
-        pass
-
 
 def beautify_factory_name(factory):
     """Returns a nice name for the specified Gst.ElementFactory instance.
@@ -372,7 +366,8 @@ class RenderingProgressDialog(GObject.Object):
         self.main_render_dialog.window.show()
 
     def _playRenderedFileButtonClickedCb(self, unused_button):
-        subprocess.Popen(["xdg-open", self.main_render_dialog.outfile])
+        uri = Gst.filename_to_uri(self.main_render_dialog.outfile)
+        Gio.AppInfo.launch_default_for_uri(uri, None)
 
 
 class RenderDialog(Loggable):
@@ -678,7 +673,7 @@ class RenderDialog(Loggable):
         Returns:
             str: A human-readable (ex: "14 MB") estimate for the file size.
         """
-        if not self.current_position or self.current_position == 0:
+        if not self.current_position:
             return None
 
         current_filesize = os.stat(path_from_uri(self.outfile)).st_size
@@ -918,7 +913,7 @@ class RenderDialog(Loggable):
                     self._factory_formats[encoder_string] = fmt
                     break
 
-        self.app.gui.timeline_ui.zoomFit()
+        self.app.gui.timeline_ui.timeline.set_best_zoom_ratio()
         self.project.set_rendering(True)
         self._pipeline.set_render_settings(
             self.outfile, self.project.container_profile)
@@ -950,21 +945,22 @@ class RenderDialog(Loggable):
     # Periodic (timer) callbacks
     def _updateTimeEstimateCb(self):
         if self._rendering_is_paused:
-            return True  # Do nothing until we resume rendering
-        elif self._is_rendering:
-            timediff = time.time() - \
-                self._time_started - self._time_spent_paused
-            length = self.project.ges_timeline.props.duration
-            totaltime = (timediff * float(length) /
-                         float(self.current_position)) - timediff
-            time_estimate = beautify_ETA(int(totaltime * Gst.SECOND))
-            if time_estimate:
-                self.progress.updateProgressbarETA(time_estimate)
+            # Do nothing until we resume rendering
+            return True
+        if self._is_rendering:
+            if self.current_position:
+                timediff = time.time() - self._time_started - self._time_spent_paused
+                length = self.project.ges_timeline.props.duration
+                estimated_time = timediff * length / self.current_position
+                remaining_time = estimated_time - timediff
+                estimate = beautify_ETA(int(remaining_time * Gst.SECOND))
+                if estimate:
+                    self.progress.updateProgressbarETA(estimate)
             return True
         else:
             self._timeEstimateTimer = None
             self.debug("Stopping the ETA timer")
-            return False  # Stop the timer
+            return False
 
     def _updateFilesizeEstimateCb(self):
         if self._rendering_is_paused:
