@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/transitions.py
-#
 # Copyright (c) 2012, Jean-François Fortin Tam <nekohayo@gmail.com>
 #
 # This program is free software; you can redistribute it and/or
@@ -19,19 +16,19 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
 import os
+from gettext import gettext as _
 
+from gi.repository import GdkPixbuf
 from gi.repository import GES
 from gi.repository import GLib
-from gi.repository import GObject
 from gi.repository import Gtk
-from gi.repository import GdkPixbuf
-
-from gettext import gettext as _
 
 from pitivi.configure import get_pixmap_dir
 from pitivi.utils.loggable import Loggable
+from pitivi.utils.misc import disconnectAllByFunc
+from pitivi.utils.ui import fix_infobar
+from pitivi.utils.ui import PADDING
 from pitivi.utils.ui import SPACING
 
 
@@ -40,16 +37,19 @@ from pitivi.utils.ui import SPACING
  COL_DESC_TEXT,
  COL_ICON) = list(range(4))
 
+BORDER_LOOP_THRESHOLD = 50000
 
-class TransitionsListWidget(Gtk.VBox, Loggable):
-    """
-    Widget for configuring the selected transition.
 
-    @type app: L{Pitivi}
+class TransitionsListWidget(Gtk.Box, Loggable):
+    """Widget for configuring the selected transition.
+
+    Attributes:
+        app (Pitivi): The app.
+        element (GES.VideoTransition): The transition being configured.
     """
 
     def __init__(self, app):
-        Gtk.VBox.__init__(self)
+        Gtk.Box.__init__(self)
         Loggable.__init__(self)
 
         self.app = app
@@ -57,43 +57,51 @@ class TransitionsListWidget(Gtk.VBox, Loggable):
         self._pixdir = os.path.join(get_pixmap_dir(), "transitions")
         icon_theme = Gtk.IconTheme.get_default()
         self._question_icon = icon_theme.load_icon("dialog-question", 48, 0)
+        self.set_orientation(Gtk.Orientation.VERTICAL)
+        # Whether a child widget has the focus.
+        self.container_focused = False
 
-        #Tooltip handling
+        # Tooltip handling
         self._current_transition_name = None
         self._current_tooltip_icon = None
 
-        #Searchbox
-        self.searchbar = Gtk.HBox()
-        self.searchbar.set_border_width(3)  # Prevents being flush against the notebook
+        # Searchbox
+        self.searchbar = Gtk.Box()
+        self.searchbar.set_orientation(Gtk.Orientation.HORIZONTAL)
+        # Prevents being flush against the notebook
+        self.searchbar.set_border_width(3)
         self.searchEntry = Gtk.Entry()
-        self.searchEntry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, "edit-clear-symbolic")
+        self.searchEntry.set_icon_from_icon_name(
+            Gtk.EntryIconPosition.SECONDARY, "edit-clear-symbolic")
         self.searchEntry.set_placeholder_text(_("Search..."))
         self.searchbar.pack_end(self.searchEntry, True, True, 0)
 
-        self.props_widgets = Gtk.VBox()
-        borderTable = Gtk.Table(n_rows=2, n_columns=3)
+        self.props_widgets = Gtk.Grid()
+        self.props_widgets.props.margin = PADDING
+        self.props_widgets.props.column_spacing = SPACING
 
-        self.border_mode_normal = Gtk.RadioButton(group=None, label=_("Normal"))
-        self.border_mode_loop = Gtk.RadioButton(group=self.border_mode_normal, label=_("Loop"))
+        self.border_mode_normal = Gtk.RadioButton(
+            group=None, label=_("Normal"))
         self.border_mode_normal.set_active(True)
-        self.borderScale = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, None)
-        self.borderScale.set_draw_value(False)
+        self.props_widgets.attach(self.border_mode_normal, 0, 0, 1, 1)
 
-        borderTable.attach(self.border_mode_normal, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.FILL)
-        borderTable.attach(self.border_mode_loop, 1, 2, 0, 1, xoptions=Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.FILL)
-        # The ypadding is a hack to make the slider widget align with the radiobuttons.
-        borderTable.attach(self.borderScale, 2, 3, 0, 2, ypadding=SPACING * 2)
+        self.border_mode_loop = Gtk.RadioButton(
+            group=self.border_mode_normal, label=_("Loop"))
+        self.props_widgets.attach(self.border_mode_loop, 0, 1, 1, 1)
+
+        self.border_scale = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, None)
+        self.border_scale.set_draw_value(False)
+        self.props_widgets.attach(self.border_scale, 1, 0, 1, 2)
 
         self.invert_checkbox = Gtk.CheckButton(label=_("Reverse direction"))
-        self.invert_checkbox.set_border_width(SPACING)
-
-        self.props_widgets.add(borderTable)
-        self.props_widgets.add(self.invert_checkbox)
+        self.invert_checkbox.props.margin_top = SPACING
+        self.props_widgets.attach(self.invert_checkbox, 1, 2, 1, 1)
 
         # Set the default values
-        self._borderTypeChangedCb()
+        self.__updateBorderScale()
 
         self.infobar = Gtk.InfoBar()
+        fix_infobar(self.infobar)
         self.infobar.props.message_type = Gtk.MessageType.OTHER
         txtlabel = Gtk.Label()
         txtlabel.set_line_wrap(True)
@@ -110,32 +118,27 @@ class TransitionsListWidget(Gtk.VBox, Loggable):
         # FIXME: the "never" horizontal scroll policy in GTK2 messes up iconview
         # Re-enable this when we switch to GTK3
         # See also http://python.6.n6.nabble.com/Cannot-shrink-width-of-scrolled-textview-tp1945060.html
-        #self.iconview_scrollwin.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        # self.iconview_scrollwin.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
 
         self.iconview = Gtk.IconView(model=self.storemodel)
         self.iconview.set_pixbuf_column(COL_ICON)
         # We don't show text because we have a searchbar and the names are ugly
-        #self.iconview.set_text_column(COL_NAME_TEXT)
+        # self.iconview.set_text_column(COL_NAME_TEXT)
         self.iconview.set_item_width(48 + 10)
         self.iconview_scrollwin.add(self.iconview)
         self.iconview.set_property("has_tooltip", True)
 
         self.searchEntry.connect("changed", self._searchEntryChangedCb)
         self.searchEntry.connect("icon-press", self._searchEntryIconClickedCb)
-        self.iconview.connect("selection-changed", self._transitionSelectedCb)
         self.iconview.connect("query-tooltip", self._queryTooltipCb)
-        self.borderScale.connect("value-changed", self._borderScaleCb)
-        self.invert_checkbox.connect("toggled", self._invertCheckboxCb)
-        self.border_mode_normal.connect("released", self._borderTypeChangedCb)
-        self.border_mode_loop.connect("released", self._borderTypeChangedCb)
 
         # Speed-up startup by only checking available transitions on idle
         GLib.idle_add(self._loadAvailableTransitionsCb)
 
-        self.pack_start(self.infobar, False, True, 0)
-        self.pack_start(self.searchbar, False, True, 0)
+        self.pack_start(self.infobar, False, False, 0)
+        self.pack_start(self.searchbar, False, False, 0)
         self.pack_start(self.iconview_scrollwin, True, True, 0)
-        self.pack_start(self.props_widgets, False, True, 0)
+        self.pack_start(self.props_widgets, False, False, 0)
 
         # Create the filterModel for searching
         self.modelFilter = self.storemodel.filter_new()
@@ -148,53 +151,101 @@ class TransitionsListWidget(Gtk.VBox, Loggable):
         self.props_widgets.hide()
         self.searchbar.hide()
 
+    def do_set_focus_child(self, child):
+        Gtk.Box.do_set_focus_child(self, child)
+        action_log = self.app.action_log
+        if not action_log:
+            # This happens when the user is editing a transition and
+            # suddenly closes the window. Don't bother.
+            return
+        if child:
+            if not self.container_focused:
+                self.container_focused = True
+                action_log.begin("Change transaction")
+        else:
+            if self.container_focused:
+                self.container_focused = False
+                action_log.commit("Change transaction")
+
+    def __connectUi(self):
+        self.iconview.connect("selection-changed", self._transitionSelectedCb)
+        self.border_scale.connect("value-changed", self._borderScaleCb)
+        self.invert_checkbox.connect("toggled", self._invertCheckboxCb)
+        self.border_mode_normal.connect("released", self._borderTypeChangedCb)
+        self.border_mode_loop.connect("released", self._borderTypeChangedCb)
+        self.element.connect("notify::border", self.__updated_cb)
+        self.element.connect("notify::invert", self.__updated_cb)
+        self.element.connect("notify::transition-type", self.__updated_cb)
+
+    def __updated_cb(self, element, unused_param):
+        self._update_ui()
+
+    def __disconnectUi(self):
+        self.iconview.disconnect_by_func(self._transitionSelectedCb)
+        self.border_scale.disconnect_by_func(self._borderScaleCb)
+        self.invert_checkbox.disconnect_by_func(self._invertCheckboxCb)
+        self.border_mode_normal.disconnect_by_func(self._borderTypeChangedCb)
+        self.border_mode_loop.disconnect_by_func(self._borderTypeChangedCb)
+        disconnectAllByFunc(self.element, self.__updated_cb)
+
 # UI callbacks
 
-    def _transitionSelectedCb(self, unused_event):
+    def _transitionSelectedCb(self, unused_widget):
         transition_asset = self.getSelectedItem()
         if not transition_asset:
             # The user clicked between icons
             return False
 
-        self.debug("New transition type selected: %s", transition_asset.get_id())
+        self.debug(
+            "New transition type selected: %s", transition_asset.get_id())
         if transition_asset.get_id() == "crossfade":
             self.props_widgets.set_sensitive(False)
         else:
             self.props_widgets.set_sensitive(True)
 
         self.element.get_parent().set_asset(transition_asset)
-        self.app.project_manager.current_project.seeker.flush(True)
+        self.app.write_action("element-set-asset",
+            asset_id=transition_asset.get_id(),
+            element_name=self.element.get_name())
+        self.app.project_manager.current_project.pipeline.flushSeek()
 
         return True
 
-    def _borderScaleCb(self, range_changed):
-        value = range_changed.get_value()
+    def _borderScaleCb(self, widget):
+        value = widget.get_value()
         self.debug("User changed the border property to %s", value)
         self.element.set_border(int(value))
-        self.app.project_manager.current_project.seeker.flush(True)
+        self.app.project_manager.current_project.pipeline.flushSeek()
 
     def _invertCheckboxCb(self, widget):
         value = widget.get_active()
         self.debug("User changed the invert property to %s", value)
         self.element.set_inverted(value)
-        self.app.project_manager.current_project.seeker.flush()
+        self.app.project_manager.current_project.pipeline.flushSeek()
 
-    def _borderTypeChangedCb(self, widget=None):
-        """
-        The "border" property in gstreamer is unlimited, but if you go over
-        25 thousand it "loops" the transition instead of smoothing it.
-        """
-        if widget == self.border_mode_loop:
-            self.borderScale.set_range(50000, 500000)
-            self.borderScale.clear_marks()
-            self.borderScale.add_mark(50000, Gtk.PositionType.BOTTOM, _("Slow"))
-            self.borderScale.add_mark(200000, Gtk.PositionType.BOTTOM, _("Fast"))
-            self.borderScale.add_mark(500000, Gtk.PositionType.BOTTOM, _("Epileptic"))
+    def _borderTypeChangedCb(self, widget):
+        self.__updateBorderScale(widget == self.border_mode_loop)
+
+    def __updateBorderScale(self, loop=False, border=None):
+        # The "border" property in gstreamer is unlimited, but if you go over
+        # 25 thousand it "loops" the transition instead of smoothing it.
+        if border is not None:
+            loop = border >= BORDER_LOOP_THRESHOLD
+        if loop:
+            self.border_scale.set_range(50000, 500000)
+            self.border_scale.clear_marks()
+            self.border_scale.add_mark(
+                50000, Gtk.PositionType.BOTTOM, _("Slow"))
+            self.border_scale.add_mark(
+                200000, Gtk.PositionType.BOTTOM, _("Fast"))
+            self.border_scale.add_mark(
+                500000, Gtk.PositionType.BOTTOM, _("Epileptic"))
         else:
-            self.borderScale.set_range(0, 25000)
-            self.borderScale.clear_marks()
-            self.borderScale.add_mark(0, Gtk.PositionType.BOTTOM, _("Sharp"))
-            self.borderScale.add_mark(25000, Gtk.PositionType.BOTTOM, _("Smooth"))
+            self.border_scale.set_range(0, 25000)
+            self.border_scale.clear_marks()
+            self.border_scale.add_mark(0, Gtk.PositionType.BOTTOM, _("Sharp"))
+            self.border_scale.add_mark(
+                25000, Gtk.PositionType.BOTTOM, _("Smooth"))
 
     def _searchEntryChangedCb(self, unused_entry):
         self.modelFilter.refilter()
@@ -202,111 +253,67 @@ class TransitionsListWidget(Gtk.VBox, Loggable):
     def _searchEntryIconClickedCb(self, entry, unused, unused_1):
         entry.set_text("")
 
-# GES callbacks
-
-    def _transitionTypeChangedCb(self, element, unused_prop):
-        try:
-            self.iconview.disconnect_by_func(self._transitionSelectedCb)
-        except TypeError:
-            pass
-        finally:
-            self.selectTransition(element.get_asset())
-            self.iconview.connect("button-release-event", self._transitionSelectedCb)
-
-    def _borderChangedCb(self, element, unused_prop):
-        """
-        The "border" transition property changed in the backend. Update the UI.
-        """
-        value = element.get_border()
-        try:
-            self.borderScale.disconnect_by_func(self._borderScaleCb)
-        except TypeError:
-            pass
-        finally:
-            self.borderScale.set_value(float(value))
-            self.borderScale.connect("value-changed", self._borderScaleCb)
-
-    def _invertChangedCb(self, element, unused_prop):
-        """
-        The "invert" transition property changed in the backend. Update the UI.
-        """
-        value = element.is_inverted()
-        try:
-            self.invert_checkbox.disconnect_by_func(self._invertCheckboxCb)
-        except TypeError:
-            pass
-        finally:
-            self.invert_checkbox.set_active(value)
-            self.invert_checkbox.connect("toggled", self._invertCheckboxCb)
-
 # UI methods
 
     def _loadAvailableTransitionsCb(self):
-        """
-        Get the list of transitions from GES and load the associated thumbnails.
-        """
+        """Loads the transitions types and icons into the storemodel."""
         for trans_asset in GES.list_assets(GES.BaseTransitionClip):
             trans_asset.icon = self._getIcon(trans_asset.get_id())
             self.storemodel.append([trans_asset,
                                     str(trans_asset.get_id()),
-                                    str(trans_asset.get_meta(GES.META_DESCRIPTION)),
+                                    str(trans_asset.get_meta(
+                                        GES.META_DESCRIPTION)),
                                     trans_asset.icon])
 
         # Now that the UI is fully ready, enable searching
         self.modelFilter.set_visible_func(self._setRowVisible, data=None)
         # Alphabetical/name sorting instead of based on the ID number
-        self.storemodel.set_sort_column_id(COL_NAME_TEXT, Gtk.SortType.ASCENDING)
+        self.storemodel.set_sort_column_id(
+            COL_NAME_TEXT, Gtk.SortType.ASCENDING)
 
     def activate(self, element):
-        """
-        Hide the infobar and show the transitions UI.
-        """
+        """Hides the infobar and shows the transitions UI."""
+        if isinstance(element, GES.AudioTransition):
+            return
         self.element = element
-        self.element.connect("notify::border", self._borderChangedCb)
-        self.element.connect("notify::invert", self._invertChangedCb)
-        self.element.connect("notify::type", self._transitionTypeChangedCb)
-        transition_asset = element.get_parent().get_asset()
-        if transition_asset.get_id() == "crossfade":
-            self.props_widgets.set_sensitive(False)
-        else:
-            self.props_widgets.set_sensitive(True)
+        self._update_ui()
         self.iconview.show_all()
         self.props_widgets.show_all()
         self.searchbar.show_all()
-        self.selectTransition(transition_asset)
-        self.app.gui.switchContextTab(element)
+        self.__connectUi()
         # We REALLY want the infobar to be hidden as space is really constrained
         # and yet GTK 3.10 seems to be racy in showing/hiding infobars, so
         # this must happen *after* the tab has been made visible/switched to:
         self.infobar.hide()
 
-    def selectTransition(self, transition_asset):
-        """
-        For a given transition type, select it in the iconview if available.
-        """
+    def _update_ui(self):
+        transition_type = self.element.get_transition_type()
+        self.props_widgets.set_sensitive(
+            transition_type != GES.VideoStandardTransitionType.CROSSFADE)
+        self.__select_transition(transition_type)
+        border = self.element.get_border()
+        self.__updateBorderScale(border=border)
+        self.border_scale.set_value(border)
+        self.invert_checkbox.set_active(self.element.is_inverted())
+        loop = border >= BORDER_LOOP_THRESHOLD
+        if loop:
+            self.border_mode_loop.activate()
+        else:
+            self.border_mode_normal.activate()
+
+    def __select_transition(self, transition_type):
+        """Selects the specified transition type in the iconview."""
         model = self.iconview.get_model()
         for row in model:
-            if transition_asset == row[COL_TRANSITION_ASSET]:
+            asset = row[COL_TRANSITION_ASSET]
+            if transition_type.value_nick == asset.get_id():
                 path = model.get_path(row.iter)
                 self.iconview.select_path(path)
                 self.iconview.scroll_to_path(path, False, 0, 0)
 
     def deactivate(self):
-        """
-        Show the infobar and hide the transitions UI.
-        """
-        try:
-            self.element.disconnect_by_func(self._borderChangedCb)
-            self.element.disconnect_by_func(self._invertChangedCb)
-            self.element.disconnect_by_func(self._transitionTypeChangedCb)
-        except TypeError:
-            pass
-        except AttributeError:
-            # This happens when selecting a normal track object before any
-            # transition object has been created. Normal track objects don't
-            # have these signals, so we just ignore them. Anyway, we just want
-            # to deactivate the UI now.
-            pass
+        """Shows the infobar and hides the transitions UI."""
+        self.__disconnectUi()
         self.iconview.unselect_all()
         self.iconview.hide()
         self.props_widgets.hide()
@@ -314,19 +321,18 @@ class TransitionsListWidget(Gtk.VBox, Loggable):
         self.infobar.show()
 
     def _getIcon(self, transition_nick):
-        """
-        If available, return an icon pixbuf for a given transition nickname.
-        """
+        """Gets an icon pixbuf for the specified transition nickname."""
         name = transition_nick + ".png"
-        icon = None
         try:
-            icon = GdkPixbuf.Pixbuf.new_from_file(os.path.join(self._pixdir, name))
+            icon = GdkPixbuf.Pixbuf.new_from_file(
+                os.path.join(self._pixdir, name))
         except:
             icon = self._question_icon
         return icon
 
     def _queryTooltipCb(self, view, x, y, keyboard_mode, tooltip):
-        is_row, x, y, model, path, iter_ = view.get_tooltip_context(x, y, keyboard_mode)
+        is_row, x, y, model, path, iter_ = view.get_tooltip_context(
+            x, y, keyboard_mode)
         if not is_row:
             return False
 
@@ -352,9 +358,7 @@ class TransitionsListWidget(Gtk.VBox, Loggable):
         return self.modelFilter[path[0]][COL_TRANSITION_ASSET]
 
     def _setRowVisible(self, model, iter, unused_data):
-        """
-        Filters the icon view depending on the search results
-        """
+        """Filters the icon view to show only the search results."""
         text = self.searchEntry.get_text().lower()
         return text in model.get_value(iter, COL_DESC_TEXT).lower() or\
             text in model.get_value(iter, COL_NAME_TEXT).lower()
