@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/utils/widgets.py
-#
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
 # This program is free software; you can redistribute it and/or
@@ -19,42 +16,38 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
-"""
-A collection of helper classes and routines for:
-    * dynamically creating user interfaces
-    * Creating UI from GstElement-s
-"""
-
+"""Classes and routines for creating widgets from `Gst.Element`s."""
 import math
 import os
 import re
-import sys
-
-from gi.repository import Gtk
-from gi.repository import Gdk
-from gi.repository import Gst
-from gi.repository import GES
-from gi.repository import Pango
-from gi.repository import GObject
-
 from gettext import gettext as _
 
-from pitivi.utils.loggable import Loggable
+from gi.repository import Gdk
+from gi.repository import GES
+from gi.repository import GLib
+from gi.repository import GObject
+from gi.repository import Gst
+from gi.repository import GstController
+from gi.repository import Gtk
+from gi.repository import Pango
+
 from pitivi.configure import get_ui_dir
-from pitivi.utils.ui import beautify_length, \
-    unpack_color, pack_color_32, pack_color_64, \
-    time_to_string, SPACING, CONTROL_WIDTH
+from pitivi.utils.loggable import Loggable
 from pitivi.utils.timeline import Zoomable
+from pitivi.utils.ui import beautify_length
+from pitivi.utils.ui import disable_scroll
+from pitivi.utils.ui import pack_color_32
+from pitivi.utils.ui import pack_color_64
+from pitivi.utils.ui import SPACING
+from pitivi.utils.ui import time_to_string
+from pitivi.utils.ui import unpack_color
 
 
 ZOOM_SLIDER_PADDING = SPACING * 4 / 5
 
 
 class DynamicWidget(object):
-
-    """An interface which provides a uniform way to get, set, and observe
-    widget properties"""
+    """Abstract widget providing a way to get, set and observe properties."""
 
     def __init__(self, default):
         self.default = default
@@ -65,7 +58,7 @@ class DynamicWidget(object):
     def setWidgetValue(self, value):
         raise NotImplementedError
 
-    def getWidgetValue(self, value):
+    def getWidgetValue(self):
         raise NotImplementedError
 
     def getWidgetDefault(self):
@@ -77,19 +70,28 @@ class DynamicWidget(object):
 
 
 class DefaultWidget(Gtk.Label):
-
     """When all hope fails...."""
 
     def __init__(self, *unused, **unused_kwargs):
         Gtk.Label.__init__(self, _("Implement Me"))
+        self.props.halign = Gtk.Align.START
+
+    def connectValueChanged(self, callback, *args):
+        pass
+
+    def setWidgetValue(self, value):
+        pass
+
+    def getWidgetValue(self):
+        pass
 
     def setWidgetToDefault(self):
         pass
 
 
 class TextWidget(Gtk.Box, DynamicWidget):
+    """Widget for entering text.
 
-    """
     A Gtk.Entry which emits a "value-changed" signal only when its input is
     valid (matches the provided regex). If the input is invalid, a warning
     icon is displayed.
@@ -105,9 +107,6 @@ class TextWidget(Gtk.Box, DynamicWidget):
         "activate": (GObject.SignalFlags.RUN_LAST, None, (),)
     }
 
-    __INVALID__ = Gdk.Color(0xFFFF, 0, 0)
-    __NORMAL__ = Gdk.Color(0, 0, 0)
-
     def __init__(self, matches=None, choices=None, default=None):
         if not default:
             # In the case of text widgets, a blank default is an empty string
@@ -123,25 +122,26 @@ class TextWidget(Gtk.Box, DynamicWidget):
             self.combo = Gtk.ComboBoxText.new_with_entry()
             self.text = self.combo.get_child()
             self.combo.show()
-            self.pack_start(self.combo, expand=True, fill=True, padding=0)
+            disable_scroll(self.combo)
+            self.pack_start(self.combo, expand=False, fill=False, padding=0)
             for choice in choices:
                 self.combo.append_text(choice)
         else:
             self.text = Gtk.Entry()
             self.text.show()
-            self.pack_start(self.text, expand=True, fill=True, padding=0)
+            self.pack_start(self.text, expand=False, fill=False, padding=0)
         self.matches = None
         self.last_valid = None
         self.valid = False
         self.send_signal = True
-        self.text.connect("changed", self._textChanged)
-        self.text.connect("activate", self._activateCb)
+        self.text.connect("changed", self.__text_changed_cb)
+        self.text.connect("activate", self.__activate_cb)
         if matches:
             if type(matches) is str:
                 self.matches = re.compile(matches)
             else:
                 self.matches = matches
-            self._textChanged(None)
+            self.__text_changed_cb(None)
 
     def connectValueChanged(self, callback, *args):
         return self.connect("value-changed", callback, *args)
@@ -159,7 +159,7 @@ class TextWidget(Gtk.Box, DynamicWidget):
         for choice in choices:
             self.combo.append_text(choice)
 
-    def _textChanged(self, unused_widget):
+    def __text_changed_cb(self, unused_widget):
         text = self.text.get_text()
         if self.matches:
             if self._filter(text):
@@ -178,14 +178,7 @@ class TextWidget(Gtk.Box, DynamicWidget):
 
         self.send_signal = True
 
-    def _activateCb(self, unused_widget):
-        """
-        Similar to _textChanged, to account for the case where we connect to
-        the "activate" signal instead of "text-changed".
-
-        We don't need to set the icons or anything like that, as _textChanged
-        does it already.
-        """
+    def __activate_cb(self, unused_widget):
         if self.matches and self.send_signal:
             self.emit("activate")
 
@@ -201,10 +194,16 @@ class TextWidget(Gtk.Box, DynamicWidget):
 
 
 class NumericWidget(Gtk.Box, DynamicWidget):
+    """Widget for entering a number.
 
-    """An horizontal Gtk.Scale and a Gtk.SpinButton which share an adjustment.
+    Contains both a Gtk.Scale and a Gtk.SpinButton for adjusting the value.
     The SpinButton is always displayed, while the Scale only appears if both
-    lower and upper bounds are defined"""
+    lower and upper bounds are defined.
+
+    Args:
+        upper (Optional[int]): The upper limit for this widget.
+        lower (Optional[int]): The lower limit for this widget.
+    """
 
     def __init__(self, upper=None, lower=None, default=None):
         Gtk.Box.__init__(self)
@@ -212,15 +211,31 @@ class NumericWidget(Gtk.Box, DynamicWidget):
 
         self.set_orientation(Gtk.Orientation.HORIZONTAL)
         self.set_spacing(SPACING)
-        self.adjustment = Gtk.Adjustment()
-        self.upper = upper
-        self.lower = lower
         self._type = None
-        if (lower is not None and upper is not None) and (lower > -5000 and upper < 5000):
+
+        reasonable_limit = 5000
+        with_slider = (lower is not None and lower > -reasonable_limit and
+            upper is not None and upper < reasonable_limit)
+
+        self.adjustment = Gtk.Adjustment()
+        # Limit the limits, otherwise the widget appears huge.
+        # Workaround https://bugzilla.gnome.org/show_bug.cgi?id=727294
+        # If these hardcoded limits are a problem, the fix should be passing
+        # the proper limits using the upper and lower parameters.
+        if upper is None:
+            upper = GLib.MAXINT16
+        if lower is None:
+            lower = GLib.MININT16
+        self.adjustment.props.lower = max(GLib.MININT16, lower)
+        self.adjustment.props.upper = min(upper, GLib.MAXINT16)
+
+        if with_slider:
             self.slider = Gtk.Scale.new(
                 Gtk.Orientation.HORIZONTAL, self.adjustment)
-            self.pack_start(self.slider, expand=True, fill=True, padding=0)
+            self.pack_end(self.slider, expand=False, fill=False, padding=0)
             self.slider.show()
+            disable_scroll(self.slider)
+            self.slider.set_size_request(width=100, height=-1)
             self.slider.props.draw_value = False
             # Abuse GTK3's progressbar "fill level" feature to provide
             # a visual indication of the default value on property sliders.
@@ -229,16 +244,10 @@ class NumericWidget(Gtk.Box, DynamicWidget):
                 self.slider.set_fill_level(float(default))
                 self.slider.set_show_fill_level(True)
 
-        if upper is None:
-            upper = GObject.G_MAXDOUBLE
-        if lower is None:
-            lower = GObject.G_MINDOUBLE
-        self.adjustment.props.lower = lower
-        self.adjustment.props.upper = upper
         self.spinner = Gtk.SpinButton(adjustment=self.adjustment)
-        self.pack_end(self.spinner, fill=True,
-                      expand=not hasattr(self, 'slider'), padding=0)
+        self.pack_start(self.spinner, expand=False, fill=False, padding=0)
         self.spinner.show()
+        disable_scroll(self.spinner)
 
     def connectValueChanged(self, callback, *args):
         self.adjustment.connect("value-changed", callback, *args)
@@ -254,29 +263,26 @@ class NumericWidget(Gtk.Box, DynamicWidget):
         if self._type is None:
             self._type = type_
 
-        if type_ == int or type_ == int:
-            minimum, maximum = (-sys.maxsize, sys.maxsize)
+        if type_ == int:
             step = 1.0
             page = 10.0
         elif type_ == float:
-            minimum, maximum = (GObject.G_MINDOUBLE, GObject.G_MAXDOUBLE)
             step = 0.01
             page = 0.1
             self.spinner.props.digits = 2
-        if self.lower is not None:
-            minimum = self.lower
-        if self.upper is not None:
-            maximum = self.upper
-        self.adjustment.configure(value, minimum, maximum, step, page, 0)
-        self.spinner.set_adjustment(self.adjustment)
+        else:
+            raise Exception('Unsupported property type: %s' % type_)
+        lower = min(self.adjustment.props.lower, value)
+        upper = max(self.adjustment.props.upper, value)
+        self.adjustment.configure(value, lower, upper, step, page, 0)
 
 
 class TimeWidget(TextWidget, DynamicWidget):
+    """Widget for entering a time value.
 
+    Accepts timecode formats or a frame number (integer).
     """
-    A widget that contains a time in nanoseconds. Accepts timecode formats
-    or a frame number (integer).
-    """
+
     # The "frame number" match rule is ^([0-9]+)$ (with a + to require 1 digit)
     # The "timecode" rule is ^([0-9]:[0-5][0-9]:[0-5][0-9])\.[0-9][0-9][0-9]$"
     # Combining the two, we get:
@@ -311,7 +317,6 @@ class TimeWidget(TextWidget, DynamicWidget):
             # We were given a frame number. Convert from the project framerate.
             frame_no = int(timecode)
             nanosecs = frame_no / float(self._framerate) * Gst.SECOND
-        # The seeker won't like floating point nanoseconds!
         return int(nanosecs)
 
     def setWidgetValue(self, timeNanos, send_signal=True):
@@ -334,8 +339,7 @@ class TimeWidget(TextWidget, DynamicWidget):
 
 
 class FractionWidget(TextWidget, DynamicWidget):
-
-    """A Gtk.ComboBoxEntry """
+    """Widget for entering a fraction."""
 
     fraction_regex = re.compile(
         "^([0-9]*(\.[0-9]+)?)(([:/][0-9]*(\.[0-9]+)?)|M)?$")
@@ -420,8 +424,7 @@ class FractionWidget(TextWidget, DynamicWidget):
 
 
 class ToggleWidget(Gtk.CheckButton, DynamicWidget):
-
-    """A Gtk.CheckButton which supports the DynamicWidget interface."""
+    """Widget for entering an on/off value."""
 
     def __init__(self, default=None):
         Gtk.CheckButton.__init__(self)
@@ -438,10 +441,7 @@ class ToggleWidget(Gtk.CheckButton, DynamicWidget):
 
 
 class ChoiceWidget(Gtk.Box, DynamicWidget):
-
-    """Abstractly, represents a choice between a list of named values. The
-    association between value names and values is arbitrary. The current
-    implementation uses a Gtk.ComboBoxText for simplicity."""
+    """Widget for making a choice between a list of named values."""
 
     def __init__(self, choices, default=None):
         Gtk.Box.__init__(self)
@@ -450,9 +450,10 @@ class ChoiceWidget(Gtk.Box, DynamicWidget):
         self.values = None
         self.set_orientation(Gtk.Orientation.HORIZONTAL)
         self.contents = Gtk.ComboBoxText()
-        self.pack_start(self.contents, expand=True, fill=True, padding=0)
+        self.pack_start(self.contents, expand=False, fill=False, padding=0)
         self.setChoices(choices)
         self.contents.show()
+        disable_scroll(self.contents)
         cell = self.contents.get_cells()[0]
         cell.props.ellipsize = Pango.EllipsizeMode.END
 
@@ -482,15 +483,12 @@ class ChoiceWidget(Gtk.Box, DynamicWidget):
 
 
 class PathWidget(Gtk.FileChooserButton, DynamicWidget):
-
-    """A Gtk.FileChooserButton which supports the DynamicWidget interface."""
+    """Widget for entering a path."""
 
     __gtype_name__ = 'PathWidget'
 
     __gsignals__ = {
-        "value-changed": (GObject.SignalFlags.RUN_LAST,
-                          None,
-                          ()),
+        "value-changed": (GObject.SignalFlags.RUN_LAST, None, ()),
     }
 
     def __init__(self, action=Gtk.FileChooserAction.OPEN, default=None):
@@ -578,255 +576,260 @@ class FontWidget(Gtk.FontButton, DynamicWidget):
 
 
 class GstElementSettingsWidget(Gtk.Box, Loggable):
+    """Widget to modify the properties of a Gst.Element.
 
-    """
-    Widget to view/modify properties of a Gst.Element
+    Can be used to configure an effect or an encoder, etc.
+
+    Args:
+        controllable (bool): Whether the properties being controlled by
+            keyframes is allowed.
     """
 
-    def __init__(self, isControllable=True):
+    def __init__(self, controllable=True):
         Gtk.Box.__init__(self)
         Loggable.__init__(self)
         self.element = None
-        self.ignore = None
-        self.properties = None
-        self.buttons = {}
-        self.isControllable = isControllable
+        self.ignore = []
+        self.properties = {}
+        self.__controllable = controllable
         self.set_orientation(Gtk.Orientation.VERTICAL)
 
-    def resetKeyframeToggleButtons(self, widget=None):
-        """
-        Reset all the keyframe togglebuttons for all properties.
-        If a property widget is specified, reset only its keyframe togglebutton.
-        """
-        if widget:
-            # Use the dynamic widget (that has been provided as an argument)
-            # to find which of the togglebuttons is the related one.
-            self.log("Resetting one keyframe button")
-            for togglebutton in list(self.keyframeToggleButtons.keys()):
-                if self.keyframeToggleButtons[togglebutton] is widget:
-                    # The dynamic widget matches the one
-                    # related to the current to the current togglebutton
-                    togglebutton.set_label("◇")
-                    self._setKeyframeToggleButtonState(togglebutton, False)
-                    break  # Stop searching
-        else:
-            self.log("Resetting all keyframe buttons")
-            for togglebutton in list(self.keyframeToggleButtons.keys()):
-                togglebutton.set_label("◇")
-                self._setKeyframeToggleButtonState(togglebutton, False)
+    def deactivate_keyframe_toggle_buttons(self):
+        """Makes sure the keyframe togglebuttons are deactivated."""
+        self.log("Deactivating all keyframe toggle buttons")
+        for keyframe_button in self.__widgets_by_keyframe_button.keys():
+            if keyframe_button.get_active():
+                # Deactivate the button. The only effect should be that
+                # the keyframe curve will control again the default property.
+                keyframe_button.set_active(False)
+                # There can be only one active keyframes button.
+                break
 
-        effect = self.element
-        parent = effect.get_parent()
-        if not parent:
-            self.log("Effect has no parent (it has been removed?)")
-            return
+    def setElement(self, element, values={}, ignore=['name'],
+                   with_reset_button=False):
+        """Sets the element to be edited.
 
-    def setElement(self, element, properties={}, ignore=['name'],
-                   default_btn=False, use_element_props=False):
+        Args:
+            values (dict): The current values of the element props, by name.
+                If empty, the default values will be used.
+            with_reset_button (bool): Whether to show a reset button for each
+                property.
         """
-        Set given element on Widget, with optional properties
-        """
-        self.info("element: %s, use properties: %s", element, properties)
+        self.info("element: %s, use values: %s", element, values)
         self.element = element
         self.ignore = ignore
-        self.properties = {}
-        self._addWidgets(properties, default_btn, use_element_props)
+        self.__add_widgets(values, with_reset_button)
 
-    def _addWidgets(self, properties, default_btn, use_element_props):
-        """
-        Prepare a gtk table containing the property widgets of an element.
-        Each property is on a separate row of the table.
+    def __add_widgets(self, values, with_reset_button):
+        """Prepares a Gtk.Grid containing the property widgets of an element.
+
+        Each property is on a separate row.
         A row is typically a label followed by the widget and a reset button.
 
-        If there are no properties, returns a table containing the label
-        "No properties."
+        If there are no properties, returns a "No properties" label.
         """
-        self.bindings = {}
-        self.keyframeToggleButtons = {}
-        is_effect = False
-        if isinstance(self.element, GES.Effect):
-            is_effect = True
-            props = [
-                prop for prop in self.element.list_children_properties() if prop.name not in self.ignore]
+        self.properties.clear()
+        self.__bindings_by_keyframe_button = {}
+        self.__widgets_by_keyframe_button = {}
+        is_effect = isinstance(self.element, GES.Effect)
+        if is_effect:
+            props = [prop for prop in self.element.list_children_properties()
+                     if prop.name not in self.ignore]
         else:
-            props = [prop for prop in GObject.list_properties(
-                self.element) if prop.name not in self.ignore]
+            props = [prop for prop in GObject.list_properties(self.element)
+                     if prop.name not in self.ignore]
         if not props:
-            table = Gtk.Table(n_rows=1, n_columns=1)
             widget = Gtk.Label(label=_("No properties."))
-            widget.set_sensitive(False)
-            table.attach(widget, 0, 1, 0, 1, yoptions=Gtk.AttachOptions.FILL)
-            self.pack_start(table, expand=True, fill=True, padding=0)
-            self.show_all()
+            self.pack_start(widget, expand=False, fill=False, padding=0)
+            widget.show()
             return
 
-        if default_btn:
-            table = Gtk.Table(n_rows=len(props), n_columns=4)
-        else:
-            table = Gtk.Table(n_rows=len(props), n_columns=3)
+        grid = Gtk.Grid()
+        grid.props.row_spacing = SPACING
+        grid.props.column_spacing = SPACING
+        grid.props.border_width = SPACING
 
-        table.set_row_spacings(SPACING)
-        table.set_col_spacings(SPACING)
-        table.set_border_width(SPACING)
-
-        y = 0
-        for prop in props:
+        for y, prop in enumerate(props):
             # We do not know how to work with GObjects, so blacklist
             # them to avoid noise in the UI
             if (not prop.flags & GObject.PARAM_WRITABLE or
-               not prop.flags & GObject.PARAM_READABLE or
-               GObject.type_is_a(prop.value_type, GObject.Object)):
+                    not prop.flags & GObject.PARAM_READABLE or
+                    GObject.type_is_a(prop.value_type, GObject.Object)):
                 continue
 
             if is_effect:
                 result, prop_value = self.element.get_child_property(prop.name)
-                if result is False:
+                if not result:
                     self.debug(
                         "Could not get value for property: %s", prop.name)
             else:
-                if use_element_props:
+                if prop.name not in values.keys():
+                    # Use the default value.
                     prop_value = self.element.get_property(prop.name)
                 else:
-                    prop_value = properties.get(prop.name)
+                    prop_value = values[prop.name]
 
             widget = self._makePropertyWidget(prop, prop_value)
             if isinstance(widget, ToggleWidget):
                 widget.set_label(prop.nick)
-                table.attach(
-                    widget, 0, 2, y, y + 1, yoptions=Gtk.AttachOptions.FILL)
+                grid.attach(widget, 0, y, 2, 1)
             else:
-                label = Gtk.Label(label=prop.nick + ":")
+                text = _("%(preference_label)s:") % {"preference_label": prop.nick}
+                label = Gtk.Label(label=text)
                 label.set_alignment(0.0, 0.5)
-                table.attach(
-                    label, 0, 1, y, y + 1, xoptions=Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.FILL)
-                table.attach(
-                    widget, 1, 2, y, y + 1, yoptions=Gtk.AttachOptions.FILL)
-
-            if not isinstance(widget, ToggleWidget) and not isinstance(widget, ChoiceWidget) and self.isControllable:
-                button = self._getKeyframeToggleButton(prop)
-                self.keyframeToggleButtons[button] = widget
-                table.attach(
-                    button, 3, 4, y, y + 1, xoptions=Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.FILL)
+                grid.attach(label, 0, y, 1, 1)
+                grid.attach(widget, 1, y, 1, 1)
 
             if hasattr(prop, 'blurb'):
                 widget.set_tooltip_text(prop.blurb)
 
             self.properties[prop] = widget
 
+            if not self.__controllable or isinstance(widget, DefaultWidget):
+                continue
+
+            keyframe_button = None
+            if not isinstance(widget, (ToggleWidget, ChoiceWidget)):
+                res, element, pspec = self.element.lookup_child(prop.name)
+                assert res
+                binding = GstController.DirectControlBinding.new(
+                    element, prop.name,
+                    GstController.InterpolationControlSource())
+                if binding.pspec:
+                    # The prop can be controlled (keyframed).
+                    keyframe_button = self.__create_keyframe_toggle_button(prop, widget)
+                    grid.attach(keyframe_button, 2, y, 1, 1)
+
             # The "reset to default" button associated with this property
-            if default_btn:
-                widget.propName = prop.name.split("-")[0]
-
-                if self.isControllable:
-                    # If this element is controlled, the value means nothing
-                    # anymore.
-                    binding = self.element.get_control_binding(prop.name)
-                    if binding:
-                        widget.set_sensitive(False)
-                        self.bindings[widget] = binding
-                button = self._getResetToDefaultValueButton(prop, widget)
-                table.attach(
-                    button, 2, 3, y, y + 1, xoptions=Gtk.AttachOptions.FILL, yoptions=Gtk.AttachOptions.FILL)
-                self.buttons[button] = widget
-
-            y += 1
+            if with_reset_button:
+                button = self.__create_reset_to_default_button(prop, widget,
+                                                               keyframe_button)
+                grid.attach(button, 3, y, 1, 1)
 
         self.element.connect('deep-notify', self._propertyChangedCb)
-        self.pack_start(table, expand=True, fill=True, padding=0)
+        self.pack_start(grid, expand=False, fill=False, padding=0)
         self.show_all()
 
     def _propertyChangedCb(self, effect, gst_element, pspec):
+        if gst_element.get_control_binding(pspec.name):
+            self.log("%s controlled, not displaying value", pspec.name)
+            return
+
         widget = self.properties[pspec]
-        widget.setWidgetValue(self.element.get_child_property(pspec.name)[1])
+        res, value = self.element.get_child_property(pspec.name)
+        assert res
+        widget.setWidgetValue(value)
 
-    def _getKeyframeToggleButton(self, prop):
-        button = Gtk.ToggleButton()
-        button.set_label("◇")
-        button.props.focus_on_click = False  # Avoid the ugly selection outline
-        button.set_tooltip_text(_("Show keyframes for this value"))
-        button.connect('toggled', self._showKeyframesToggledCb, prop)
-        return button
+    def __create_keyframe_toggle_button(self, prop, widget):
+        keyframe_button = Gtk.ToggleButton()
+        keyframe_button.props.focus_on_click = False  # Avoid the ugly selection outline
+        keyframe_button.set_tooltip_text(_("Show keyframes for this value"))
+        keyframe_button.set_relief(Gtk.ReliefStyle.NONE)
+        keyframe_button.connect('toggled', self.__keyframes_toggled_cb, prop)
+        self.__widgets_by_keyframe_button[keyframe_button] = widget
+        prop_binding = self.element.get_control_binding(prop.name)
+        self.__bindings_by_keyframe_button[keyframe_button] = prop_binding
+        self.__display_controlled(keyframe_button, bool(prop_binding))
+        return keyframe_button
 
-    def _getResetToDefaultValueButton(self, unused_prop, widget):
+    def __create_reset_to_default_button(self, unused_prop, widget,
+                                         keyframe_button):
         icon = Gtk.Image()
         icon.set_from_icon_name("edit-clear-all-symbolic", Gtk.IconSize.MENU)
         button = Gtk.Button()
         button.add(icon)
         button.set_tooltip_text(_("Reset to default value"))
         button.set_relief(Gtk.ReliefStyle.NONE)
-        button.connect('clicked', self._defaultBtnClickedCb, widget)
+        button.connect('clicked', self.__reset_to_default_clicked_cb, widget,
+                       keyframe_button)
         return button
 
-    def _setKeyframeToggleButtonState(self, button, active_state):
-        """
-        This is meant for programmatically (un)pushing the provided keyframe
-        togglebutton, without triggering its signals.
-        """
-        self.log("Manually resetting the UI state of %s" % button)
-        button.handler_block_by_func(self._showKeyframesToggledCb)
-        button.set_active(active_state)
-        button.handler_unblock_by_func(self._showKeyframesToggledCb)
+    def __set_keyframe_active(self, toggle_button, active):
+        """Updates the specified button without triggering signals.
 
-    def _showKeyframesToggledCb(self, button, prop):
-        self.log("keyframes togglebutton clicked for %s" % prop)
-        active = button.get_active()
-        # Disable the related dynamic gst property widget
-        widget = self.keyframeToggleButtons[button]
-        widget.set_sensitive(False)
+        Args:
+            toggle_button (Gtk.ToggleButton): The toggle button enabling controlling
+                the property by keyframes.
+            active (bool): The desired status of the `toggle_button`.
+        """
+        self.log("Manually resetting the UI state of %s", toggle_button)
+        toggle_button.handler_block_by_func(self.__keyframes_toggled_cb)
+        toggle_button.set_active(active)
+        toggle_button.handler_unblock_by_func(self.__keyframes_toggled_cb)
+
+    def __display_controlled(self, toggle_button, controlled):
+        """Displays whether the prop is keyframed."""
+        widget = self.__widgets_by_keyframe_button[toggle_button]
+        # The displayed value means nothing if the prop is controlled.
+        widget.set_sensitive(not controlled)
+        # The full disc label indicates the property is controlled (keyframed).
+        toggle_button.set_label("◆" if controlled else "◇")
+
+    def __keyframes_toggled_cb(self, keyframe_button, prop):
+        widget = self.__widgets_by_keyframe_button[keyframe_button]
+        self.log("keyframes togglebutton clicked for %s", prop)
+        active = keyframe_button.get_active()
         # Now change the state of the *other* togglebuttons.
-        for togglebutton in list(self.keyframeToggleButtons.keys()):
-            if togglebutton != button:
+        for toggle_keyframe_button in self.__widgets_by_keyframe_button.keys():
+            if toggle_keyframe_button != keyframe_button:
                 # Don't use set_active directly on the buttons; doing so will
                 # fire off signals that will toggle the others/confuse the UI
-                self._setKeyframeToggleButtonState(togglebutton, False)
-        # We always set this label, since the only way to *deactivate* keyframes
-        # (not just hide them temporarily) is to use the separate reset button.
-        button.set_label("◆")
+                self.__set_keyframe_active(toggle_keyframe_button, False)
+        # The only way to *deactivate* keyframes (not just hide them) is to use
+        # the separate reset button.
+        self.__display_controlled(keyframe_button, True)
 
-        effect = self.element
+        track_element = self.__get_track_element_of_same_type(self.element)
+        if not track_element:
+            return
+
+        if active:
+            track_element.ui_element.showKeyframes(self.element, prop)
+            binding = self.element.get_control_binding(prop.name)
+            self.__bindings_by_keyframe_button[keyframe_button] = binding
+        else:
+            track_element.ui_element.showDefaultKeyframes()
+
+    def __reset_to_default_clicked_cb(self, unused_button, widget,
+                                      keyframe_button):
+        if keyframe_button:
+            # The prop is controllable (keyframmable).
+            binding = self.__bindings_by_keyframe_button.get(keyframe_button)
+            if binding:
+                # The prop has been keyframed
+                binding.props.control_source.unset_all()
+                if keyframe_button.get_active():
+                    track_element = self.__get_track_element_of_same_type(
+                        self.element)
+                    if track_element:
+                        track_element.ui_element.showDefaultKeyframes()
+                self.__set_keyframe_active(keyframe_button, False)
+                self.__display_controlled(keyframe_button, False)
+
+        widget.setWidgetToDefault()
+
+    def __get_track_element_of_same_type(self, effect):
         track_type = effect.get_track_type()
         for track_element in effect.get_parent().get_children(False):
-            if active and hasattr(track_element, "ui_element") and track_type == track_element.get_track_type():
-                track_element.ui_element.showKeyframes(effect, prop)
-                binding = self.element.get_control_binding(prop.name)
-                self.bindings[widget] = binding
-            elif hasattr(track_element, "ui_element") and track_type == track_element.get_track_type():
-                track_element.ui_element.hideKeyframes()
-
-    def _defaultBtnClickedCb(self, unused_button, widget):
-        try:
-            binding = self.bindings[widget]
-        except KeyError:
-            binding = None
-        if binding:
-            effect = self.element
-            track_type = effect.get_track_type()
-            for track_element in effect.get_parent().get_children(False):
-                if hasattr(track_element, "ui_element") and track_type == track_element.get_track_type():
-                    binding.props.control_source.unset_all()
-                    track_element.ui_element.updateKeyframes()
-
-        widget.set_sensitive(True)
-        widget.setWidgetToDefault()
-        self.resetKeyframeToggleButtons(widget)
+            if hasattr(track_element, "ui_element") and \
+                    track_element.get_track_type() == track_type:
+                return track_element
+        self.warning("Failed to find track element of type %s", track_type)
+        return None
 
     def getSettings(self, with_default=False):
-        """
-        returns the dictionnary of propertyname/propertyvalue
-        """
-        d = {}
+        """Gets a name/value dict with the properties."""
+        values = {}
         for prop, widget in self.properties.items():
             if not prop.flags & GObject.PARAM_WRITABLE:
                 continue
-            if isinstance(widget, DefaultWidget):
-                continue
             value = widget.getWidgetValue()
             if value is not None and (value != prop.default_value or with_default):
-                d[prop.name] = value
-        return d
+                values[prop.name] = value
+        return values
 
     def _makePropertyWidget(self, prop, value=None):
-        """ Creates a Widget for the specified element property """
+        """Creates a widget for the specified element property."""
         type_name = GObject.type_name(prop.value_type.fundamental)
         if type_name == "gchararray":
             widget = TextWidget(default=prop.default_value)
@@ -855,22 +858,26 @@ class GstElementSettingsWidget(Gtk.Box, Loggable):
 
         if value is None:
             value = prop.default_value
-        if value is not None and not isinstance(widget, DefaultWidget):
+        else:
             widget.setWidgetValue(value)
 
         return widget
 
 
 class GstElementSettingsDialog(Loggable):
+    """Dialog window for viewing/modifying properties of a Gst.Element."""
 
-    """
-    Dialog window for viewing/modifying properties of a Gst.Element
-    """
-
-    def __init__(self, elementfactory, properties, parent_window=None, isControllable=True):
+    def __init__(self, elementfactory, properties, parent_window=None):
         Loggable.__init__(self)
         self.debug("factory: %s, properties: %s", elementfactory, properties)
 
+        self.factory = elementfactory
+        self.element = self.factory.create("elementsettings")
+        if not self.element:
+            self.warning(
+                "Couldn't create element from factory %s", self.factory)
+        self.properties = properties
+
         self.builder = Gtk.Builder()
         self.builder.add_from_file(
             os.path.join(get_ui_dir(), "elementsettingsdialog.ui"))
@@ -878,20 +885,17 @@ class GstElementSettingsDialog(Loggable):
         self.ok_btn = self.builder.get_object("okbutton1")
 
         self.window = self.builder.get_object("dialog1")
-        self.elementsettings = GstElementSettingsWidget(isControllable)
+        self.elementsettings = GstElementSettingsWidget(controllable=False)
         self.builder.get_object("viewport1").add(self.elementsettings)
 
-        self.factory = elementfactory
-        self.element = self.factory.create("elementsettings")
-        if not self.element:
-            self.warning(
-                "Couldn't create element from factory %s", self.factory)
-        self.properties = properties
-        self._fillWindow()
+        # set title and frame label
+        self.window.set_title(
+            _("Properties for %s") % self.factory.get_longname())
+        self.elementsettings.setElement(self.element, self.properties)
 
         # Try to avoid scrolling, whenever possible.
         screen_height = self.window.get_screen().get_height()
-        contents_height = self.elementsettings.size_request().height
+        contents_height = self.elementsettings.get_preferred_size()[0].height
         maximum_contents_height = max(500, 0.7 * screen_height)
         if contents_height < maximum_contents_height:
             # The height of the content is small enough, disable the
@@ -910,14 +914,11 @@ class GstElementSettingsDialog(Loggable):
             self.window.set_transient_for(parent_window)
         self.window.show()
 
-    def _fillWindow(self):
-        # set title and frame label
-        self.window.set_title(
-            _("Properties for %s") % self.factory.get_longname())
-        self.elementsettings.setElement(self.element, self.properties)
-
     def getSettings(self):
-        """ returns the property/value dictionnary of the selected settings """
+        """Gets the settings of the `element`.
+
+        Returns:
+            dict: A property name to value map."""
         return self.elementsettings.getSettings()
 
     def _resetValuesClickedCb(self, unused_button):
@@ -931,7 +932,6 @@ class GstElementSettingsDialog(Loggable):
 class BaseTabs(Gtk.Notebook):
 
     def __init__(self, app, hide_hpaned=False):
-        """ initialize """
         Gtk.Notebook.__init__(self)
         self.set_border_width(SPACING)
 
@@ -941,7 +941,7 @@ class BaseTabs(Gtk.Notebook):
         self._createUi()
 
     def _createUi(self):
-        """ set up the gui """
+        """Sets up the GUI."""
         settings = self.get_settings()
         settings.props.gtk_dnd_drag_threshold = 1
         self.set_tab_pos(Gtk.PositionType.TOP)
@@ -1012,11 +1012,10 @@ class BaseTabs(Gtk.Notebook):
 
 
 class ZoomBox(Gtk.Grid, Zoomable):
+    """Container holding the widgets for zooming.
 
-    """
-    Container holding the widgets for zooming.
-
-    @type timeline: TimelineContainer
+    Attributes:
+        timeline (TimelineContainer): The timeline container this belongs to.
     """
 
     def __init__(self, timeline):
@@ -1031,7 +1030,7 @@ class ZoomBox(Gtk.Grid, Zoomable):
         zoom_fit_btn.set_tooltip_text(_("Zoom Fit"))
         zoom_fit_btn_grid = Gtk.Grid()
         zoom_fit_icon = Gtk.Image.new_from_icon_name(
-            "zoom-best-fit", Gtk.IconSize.BUTTON)
+            "zoom-fit-best", Gtk.IconSize.BUTTON)
         zoom_fit_btn_grid.add(zoom_fit_icon)
         zoom_fit_btn_label = Gtk.Label(label=_("Zoom"))
         zoom_fit_btn_grid.add(zoom_fit_btn_label)
@@ -1064,20 +1063,19 @@ class ZoomBox(Gtk.Grid, Zoomable):
 
         self.set_hexpand(False)
         self.set_column_spacing(ZOOM_SLIDER_PADDING)
-        self.set_size_request(CONTROL_WIDTH, -1)
         self.show_all()
 
     def _zoomAdjustmentChangedCb(self, adjustment):
         Zoomable.setZoomLevel(adjustment.get_value())
         self.timeline.app.write_action("set-zoom-level",
-                                       {"level": adjustment.get_value(),
-                                        "optional-action-type": True})
+                                       level=adjustment.get_value(),
+                                       optional_action_type=True)
 
         if self._manual_set is False:
-            self.timeline.scrollToPlayhead()
+            self.timeline.timeline.scrollToPlayhead()
 
     def _zoomFitCb(self, unused_button):
-        self.timeline.zoomFit()
+        self.timeline.timeline.set_best_zoom_ratio()
 
     def _zoomSliderScrollCb(self, unused, event):
         delta = 0
