@@ -22,34 +22,16 @@ from gi.repository import Gdk
 from gi.repository import GES
 from gi.repository import Gtk
 
-from pitivi.project import ProjectManager
-from pitivi.timeline.timeline import TimelineContainer
-from pitivi.utils import ui
-from pitivi.utils.validate import Event
+from pitivi.utils.ui import LAYER_HEIGHT
+from pitivi.utils.ui import SEPARATOR_HEIGHT
 from tests import common
+from tests.common import create_timeline_container
 
-
-SEPARATOR_HEIGHT = 4
-THIN = ui.LAYER_HEIGHT / 2
-THICK = ui.LAYER_HEIGHT
+THIN = LAYER_HEIGHT / 2
+THICK = LAYER_HEIGHT
 
 
 class BaseTestTimeline(common.TestCase):
-    def createTimeline(self):
-        app = common.create_pitivi_mock()
-        project_manager = ProjectManager(app)
-        project_manager.newBlankProject()
-        project = project_manager.current_project
-
-        timeline_container = TimelineContainer(app)
-        timeline_container.setProject(project)
-
-        timeline = timeline_container.timeline
-        timeline.get_parent = mock.MagicMock()
-
-        timeline.app.settings.leftClickAlsoSeeks = False
-
-        return timeline
 
     def addClipsSimple(self, timeline, num_clips):
         layer = timeline.ges_timeline.append_layer()
@@ -62,6 +44,7 @@ class BaseTestTimeline(common.TestCase):
 
 
 class TestLayers(BaseTestTimeline):
+
     def testDraggingLayer(self):
         self.checkGetLayerAt([THIN, THIN, THIN], 1, True,
                              [0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2])
@@ -93,8 +76,10 @@ class TestLayers(BaseTestTimeline):
                              [0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2])
 
     def checkGetLayerAt(self, heights, preferred, past_middle_when_adjacent, expectations):
-        timeline = self.createTimeline()
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
 
+        # Allocate layers
         y = 0
         for priority, height in enumerate(heights):
             ges_layer = timeline.createLayer(priority=priority)
@@ -104,12 +89,13 @@ class TestLayers(BaseTestTimeline):
             ges_layer.ui.set_allocation(rect)
             y += height + SEPARATOR_HEIGHT
 
-        ges_layers = [layer.ges_layer for layer in timeline._layers]
+        ges_layers = timeline.ges_timeline.get_layers()
         if preferred is None:
             preferred_ges_layer = None
         else:
             preferred_ges_layer = ges_layers[preferred]
-        h = [layer.get_allocation().height for layer in timeline._layers]
+        # The heights of the layers.
+        h = [ges_layer.ui.get_allocation().height for ges_layer in ges_layers]
         s = SEPARATOR_HEIGHT
 
         def assertLayerAt(ges_layer, y):
@@ -149,33 +135,134 @@ class TestLayers(BaseTestTimeline):
         assertLayerAt(ges_layers[expectations[15]], h[0] + s + h[1] + s + h[2] - 1)
 
     def testSetSeparatorsPrelight(self):
-        timeline = self.createTimeline()
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
         timeline.__on_separators = [mock.Mock()]
         timeline._setSeparatorsPrelight(False)
         self.assertEqual(len(timeline.__on_separators), 1,
                          "The separators must be forgotten only in dragEnd()")
 
+    def test_create_layer(self):
+        self.check_create_layer([0, 0, 0, 0], [3, 2, 1, 0])
+        self.check_create_layer([0, 1, 1, 1], [0, 3, 2, 1])
+        self.check_create_layer([0, 1, 1, 2], [0, 3, 1, 2])
+        self.check_create_layer([0, 1, 0, 2], [1, 3, 0, 2])
+        self.check_create_layer([0, 1, 2, 3], [0, 1, 2, 3])
+
+    def check_create_layer(self, start_priorities, expected_priorities):
+        timeline = create_timeline_container().timeline
+        ges_layers = []
+        for priority in start_priorities:
+            ges_layer = timeline.createLayer(priority)
+            self.assertEqual(ges_layer.props.priority, priority)
+            ges_layers.append(ges_layer)
+        self.check_priorities_and_positions(timeline, ges_layers, expected_priorities)
+
+    def check_priorities_and_positions(self, timeline, ges_layers,
+                                       expected_priorities):
+        layers_vbox = timeline.layout.layers_vbox
+
+        # Check the layers priorities.
+        priorities = [ges_layer.props.priority for ges_layer in ges_layers]
+        self.assertListEqual(priorities, expected_priorities)
+
+        # Check the positions of the Layer widgets.
+        positions = [layers_vbox.child_get_property(ges_layer.ui, "position")
+                     for ges_layer in ges_layers]
+        expected_positions = [priority * 2 + 1
+                              for priority in expected_priorities]
+        self.assertListEqual(positions, expected_positions, layers_vbox.get_children())
+
+        # Check the positions of the LayerControl widgets.
+        controls_vbox = timeline._layers_controls_vbox
+        positions = [controls_vbox.child_get_property(ges_layer.control_ui, "position")
+                     for ges_layer in ges_layers]
+        self.assertListEqual(positions, expected_positions)
+
+        # Check the number of the separators.
+        count = len(ges_layers) + 1
+        self.assertEqual(len(timeline._separators), count)
+        controls_separators, layers_separators = list(zip(*timeline._separators))
+
+        # Check the positions of the LayerControl separators.
+        expected_positions = [2 * index for index in range(count)]
+        positions = [layers_vbox.child_get_property(separator, "position")
+                     for separator in layers_separators]
+        self.assertListEqual(positions, expected_positions)
+
+        # Check the positions of the Layer separators.
+        positions = [controls_vbox.child_get_property(separator, "position")
+                     for separator in controls_separators]
+        self.assertListEqual(positions, expected_positions)
+
+    def test_remove_layer(self):
+        self.check_remove_layer([0, 0, 0, 0])
+        self.check_remove_layer([0, 0, 1, 0])
+        self.check_remove_layer([0, 1, 0, 0])
+        self.check_remove_layer([0, 2, 1, 0])
+        self.check_remove_layer([1, 0, 1, 0])
+        self.check_remove_layer([2, 2, 0, 0])
+        self.check_remove_layer([3, 2, 1, 0])
+
+    def check_remove_layer(self, removal_order):
+        timeline = create_timeline_container().timeline
+
+        # Add layers to remove them later.
+        ges_layers = []
+        for priority in range(len(removal_order)):
+            ges_layer = timeline.createLayer(priority)
+            ges_layers.append(ges_layer)
+
+        # Remove the layers in the specified order.
+        for priority in removal_order:
+            ges_layer = ges_layers[priority]
+            self.assertTrue(timeline.ges_timeline.remove_layer(ges_layer))
+            ges_layers.remove(ges_layer)
+            self.check_priorities_and_positions(timeline, ges_layers, list(range(len(ges_layers))))
+
+    def test_move_layer(self):
+        self.check_move_layer(0, 0, [0, 1, 2, 3, 4])
+        self.check_move_layer(0, 1, [1, 0, 2, 3, 4])
+        self.check_move_layer(0, 4, [4, 0, 1, 2, 3])
+        self.check_move_layer(2, 0, [1, 2, 0, 3, 4])
+        self.check_move_layer(2, 3, [0, 1, 3, 2, 4])
+        self.check_move_layer(4, 0, [1, 2, 3, 4, 0])
+        self.check_move_layer(4, 3, [0, 1, 2, 4, 3])
+
+    def check_move_layer(self, from_priority, to_priority, expected_priorities):
+        timeline = create_timeline_container().timeline
+
+        # Add layers to move them later.
+        ges_layers = []
+        for priority in range(len(expected_priorities)):
+            ges_layer = timeline.createLayer(priority)
+            ges_layers.append(ges_layer)
+
+        timeline.moveLayer(ges_layers[from_priority], to_priority)
+        self.check_priorities_and_positions(timeline, ges_layers, expected_priorities)
+
 
 class TestGrouping(BaseTestTimeline):
-    def groupClips(self, num_clips):
-        timeline = self.createTimeline()
+
+    def group_clips(self, timeline_container, clips):
+        timeline = timeline_container.timeline
         timeline.app.settings.leftClickAlsoSeeks = False
-        clips = self.addClipsSimple(timeline, num_clips)
 
         # Press <ctrl> so selecting in ADD mode
-        timeline.sendFakeEvent(Event(event_type=Gdk.EventType.KEY_PRESS,
-                                     keyval=Gdk.KEY_Control_L))
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        timeline_container.do_key_press_event(event)
 
         # Select the 2 clips
         for clip in clips:
-            self.toggleClipSelection(clip, expect_selected=True)
+            self.toggle_clip_selection(clip, expect_selected=True)
 
         before_grouping_timeline_group = timeline.current_group
 
         for clip in clips:
             self.assertEqual(clip.get_parent(), timeline.current_group)
 
-        timeline.parent.group_action.emit("activate", None)
+        timeline_container.group_action.emit("activate", None)
 
         self.assertNotEqual(timeline.current_group, before_grouping_timeline_group)
         for clip in clips:
@@ -192,22 +279,26 @@ class TestGrouping(BaseTestTimeline):
             self.assertTrue(clip.selected.selected)
 
         group = clips[0].get_parent()
-        self.assertEqual(len(group.get_children(False)), num_clips)
-
-        return timeline
+        self.assertEqual(len(group.get_children(False)), len(clips))
 
     def testGroup(self):
-        self.groupClips(2)
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        clips = self.addClipsSimple(timeline, 2)
+        self.group_clips(timeline_container, clips)
 
     def testGroupSelection(self):
         num_clips = 2
-        timeline = self.groupClips(num_clips)
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        clips = self.addClipsSimple(timeline, num_clips)
+        self.group_clips(timeline_container, clips)
         layer = timeline.ges_timeline.get_layers()[0]
         clips = layer.get_clips()
         self.assertEqual(len(clips), num_clips)
 
         # Deselect one grouped clip clips
-        self.toggleClipSelection(clips[0], expect_selected=False)
+        self.toggle_clip_selection(clips[0], expect_selected=False)
 
         # Make sure all the clips have been deselected
         for clip in clips:
@@ -216,11 +307,14 @@ class TestGrouping(BaseTestTimeline):
 
     def testGroupUngroup(self):
         num_clips = 2
-        timeline = self.groupClips(num_clips)
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        clips = self.addClipsSimple(timeline, num_clips)
+        self.group_clips(timeline_container, clips)
 
         self.assertEqual(len(timeline.selection.selected), num_clips)
 
-        timeline.parent.ungroup_action.emit("activate", None)
+        timeline_container.ungroup_action.emit("activate", None)
         layer = timeline.ges_timeline.get_layers()[0]
         clips = layer.get_clips()
         self.assertEqual(len(clips), num_clips)
@@ -231,15 +325,16 @@ class TestGrouping(BaseTestTimeline):
     def testGroupSplittedClipAndSelectGroup(self):
         position = 5
 
-        timeline = self.createTimeline()
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
         clips = self.addClipsSimple(timeline, 1)
-        self.toggleClipSelection(clips[0], expect_selected=True)
+        self.toggle_clip_selection(clips[0], expect_selected=True)
 
         timeline.ges_timeline.get_asset().pipeline.getPosition = mock.Mock(return_value=position)
         layer = timeline.ges_timeline.get_layers()[0]
 
         # Split
-        timeline.parent.split_action.emit("activate", None)
+        timeline_container.split_action.emit("activate", None)
         clips = layer.get_clips()
         self.assertEqual(len(clips), 2)
 
@@ -248,27 +343,28 @@ class TestGrouping(BaseTestTimeline):
         self.assertTrue(clips[0].selected.selected)
         self.assertFalse(clips[1].selected.selected)
 
-        timeline.sendFakeEvent(Event(event_type=Gdk.EventType.KEY_PRESS,
-                                     keyval=Gdk.KEY_Control_L))
-        self.toggleClipSelection(clips[1], expect_selected=True)
-        timeline.sendFakeEvent(Event(event_type=Gdk.EventType.KEY_RELEASE,
-                                     keyval=Gdk.KEY_Control_L))
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        timeline_container.do_key_press_event(event)
+        self.toggle_clip_selection(clips[1], expect_selected=True)
+        timeline_container.do_key_release_event(event)
 
         for clip in clips:
             self.assertTrue(clip.selected.selected)
 
         # Group the two parts
-        timeline.parent.group_action.emit("activate", None)
+        timeline_container.group_action.emit("activate", None)
 
-        self.toggleClipSelection(clips[1], expect_selected=True)
+        self.toggle_clip_selection(clips[1], expect_selected=True)
 
     def testUngroupClip(self):
-        timeline = self.createTimeline()
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
         ges_clip, = self.addClipsSimple(timeline, 1)
 
-        self.toggleClipSelection(ges_clip, expect_selected=True)
+        self.toggle_clip_selection(ges_clip, expect_selected=True)
 
-        timeline.parent.ungroup_action.emit("activate", None)
+        timeline_container.ungroup_action.emit("activate", None)
         layer = timeline.ges_timeline.get_layers()[0]
         ges_clip0, ges_clip1 = layer.get_clips()
 
@@ -293,32 +389,75 @@ class TestGrouping(BaseTestTimeline):
         self.assertEqual(aclip._audioSource, atrackelem)
         self.assertEqual(vclip._videoSource, vtrackelem)
 
+    def test_dragging_group_on_separator(self):
+        # Create two clips on different layers and group them.
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        clip1, = self.addClipsSimple(timeline, 1)
+        layer1 = clip1.get_layer()
+
+        # Add another clip on a new layer.
+        clip2, = self.addClipsSimple(timeline, 1)
+        self.assertEqual(len(timeline.ges_timeline.get_layers()), 2)
+
+        self.group_clips(timeline_container, [clip1, clip2])
+
+        # Click the first clip in the group.
+        with mock.patch.object(Gtk, 'get_event_widget') as get_event_widget:
+            event = mock.Mock()
+            event.x = 0
+            event.get_button.return_value = True, 1
+            get_event_widget.return_value = clip1.ui
+            timeline._button_press_event_cb(None, event)
+            self.assertIsNotNone(timeline.draggingElement)
+
+            # Move it to the right, on the separator below.
+            event = mock.Mock()
+            event.get_state.return_value = Gdk.ModifierType.BUTTON1_MASK
+            with mock.patch.object(clip1.ui, "translate_coordinates") as translate_coordinates:
+                translate_coordinates.return_value = (40, 0)
+                with mock.patch.object(timeline, "_get_layer_at") as _get_layer_at:
+                    _get_layer_at.return_value = layer1, timeline._separators[1]
+                    timeline._motion_notify_event_cb(None, event)
+            self.assertTrue(timeline.got_dragged)
+
+        # Release the mouse button.
+        event = mock.Mock()
+        event.get_button.return_value = True, 1
+        timeline._button_release_event_cb(None, event)
+        self.assertEqual(len(timeline.ges_timeline.get_layers()), 2,
+                         "No new layer should have been created")
+
 
 class TestCopyPaste(BaseTestTimeline):
+
     def copyClips(self, num_clips):
-        timeline = self.createTimeline()
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
 
         clips = self.addClipsSimple(timeline, num_clips)
 
         # Press <ctrl> so selecting in ADD mode
-        timeline.sendFakeEvent(Event(event_type=Gdk.EventType.KEY_PRESS,
-                                     keyval=Gdk.KEY_Control_L))
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        timeline_container.do_key_press_event(event)
 
         # Select the 2 clips
         for clip in clips:
-            self.toggleClipSelection(clip, expect_selected=True)
+            self.toggle_clip_selection(clip, expect_selected=True)
 
-        self.assertTrue(timeline.parent.copy_action.props.enabled)
-        self.assertFalse(timeline.parent.paste_action.props.enabled)
-        timeline.parent.copy_action.emit("activate", None)
-        self.assertTrue(timeline.parent.paste_action.props.enabled)
+        self.assertTrue(timeline_container.copy_action.props.enabled)
+        self.assertFalse(timeline_container.paste_action.props.enabled)
+        timeline_container.copy_action.emit("activate", None)
+        self.assertTrue(timeline_container.paste_action.props.enabled)
 
-        return timeline
+        return timeline_container
 
     def testCopyPaste(self):
         position = 20
 
-        timeline = self.copyClips(2)
+        timeline_container = self.copyClips(2)
+        timeline = timeline_container.timeline
 
         layer = timeline.ges_timeline.get_layers()[0]
         # Monkey patching the pipeline.getPosition method
@@ -328,7 +467,7 @@ class TestCopyPaste(BaseTestTimeline):
         clips = layer.get_clips()
         self.assertEqual(len(clips), 2)
 
-        timeline.parent.paste_action.emit("activate", None)
+        timeline_container.paste_action.emit("activate", None)
 
         n_clips = layer.get_clips()
         self.assertEqual(len(n_clips), 4)
@@ -337,3 +476,38 @@ class TestCopyPaste(BaseTestTimeline):
         self.assertEqual(len(copied_clips), 2)
         self.assertEqual(copied_clips[0].props.start, position)
         self.assertEqual(copied_clips[1].props.start, position + 10)
+
+
+class TestEditing(BaseTestTimeline):
+
+    def test_trimming_on_layer_separator(self):
+        # Create a clip
+        timeline_container = create_timeline_container()
+        timeline = timeline_container.timeline
+        clip, = self.addClipsSimple(timeline, 1)
+        layer = clip.get_layer()
+
+        # Click the right trim handle of the clip.
+        with mock.patch.object(Gtk, 'get_event_widget') as get_event_widget:
+            event = mock.Mock()
+            event.get_button.return_value = True, 1
+            get_event_widget.return_value = clip.ui.rightHandle
+            timeline._button_press_event_cb(None, event)
+            self.assertIsNotNone(timeline.draggingElement)
+
+            # Drag it to the left, on the separator below.
+            event = mock.Mock()
+            event.get_state.return_value = Gdk.ModifierType.BUTTON1_MASK
+            with mock.patch.object(clip.ui.rightHandle, "translate_coordinates") as translate_coordinates:
+                translate_coordinates.return_value = (0, 0)
+                with mock.patch.object(timeline, "_get_layer_at") as _get_layer_at:
+                    _get_layer_at.return_value = layer, timeline._separators[1]
+                    timeline._motion_notify_event_cb(None, event)
+            self.assertTrue(timeline.got_dragged)
+
+        # Release the mouse button.
+        event = mock.Mock()
+        event.get_button.return_value = True, 1
+        timeline._button_release_event_cb(None, event)
+        self.assertEqual(len(timeline.ges_timeline.get_layers()), 1,
+                         "No new layer should have been created")
