@@ -28,12 +28,12 @@ from gi.repository import Gtk
 
 from pitivi.timeline.layer import Layer
 from pitivi.timeline.timeline import Timeline
+from pitivi.timeline.timeline import TimelineContainer
 from pitivi.undo.project import AssetAddedAction
 from pitivi.undo.timeline import ClipAdded
 from pitivi.undo.timeline import ClipRemoved
 from pitivi.undo.timeline import EffectAddedAction
 from pitivi.undo.undo import PropertyChangedAction
-from pitivi.utils.ui import CONTROL_WIDTH
 from pitivi.utils.ui import LAYER_HEIGHT
 from pitivi.utils.ui import URI_TARGET_ENTRY
 from tests import common
@@ -49,6 +49,15 @@ class BaseTestUndoTimeline(TestCase):
         self.layer = self.timeline.append_layer()
         self.action_log = self.app.action_log
 
+    def setup_timeline_container(self):
+        project = self.app.project_manager.current_project
+        self.timeline_container = TimelineContainer(self.app)
+        self.timeline_container.setProject(project)
+
+        timeline = self.timeline_container.timeline
+        timeline.app.project_manager.current_project = project
+        timeline.get_parent = mock.MagicMock(return_value=self.timeline_container)
+
     def getTimelineClips(self):
         for layer in self.timeline.layers:
             for clip in layer.get_clips():
@@ -73,8 +82,7 @@ class BaseTestUndoTimeline(TestCase):
 class TestTimelineObserver(BaseTestUndoTimeline):
 
     def test_layer_removed(self):
-        timeline_ui = Timeline(container=None, app=None)
-        timeline_ui.setProject(self.app.project_manager.current_project)
+        self.setup_timeline_container()
 
         layer1 = self.layer
         layer2 = self.timeline.append_layer()
@@ -82,23 +90,68 @@ class TestTimelineObserver(BaseTestUndoTimeline):
 
         self.assertEqual([layer1, layer2, layer3], self.timeline.get_layers())
         self.assertEqual([l.props.priority for l in [layer1, layer2, layer3]],
-                        list(range(3)))
+                         list(range(3)))
 
         with self.action_log.started("layer removed"):
             self.timeline.remove_layer(layer2)
 
         self.assertEqual([layer1, layer3], self.timeline.get_layers())
         self.assertEqual([l.props.priority for l in [layer1, layer3]],
-                        list(range(2)))
+                         list(range(2)))
 
         self.action_log.undo()
         self.assertEqual([layer1, layer2, layer3], self.timeline.get_layers())
         self.assertEqual([l.props.priority for l in [layer1, layer2, layer3]],
-                        list(range(3)))
+                         list(range(3)))
         self.action_log.redo()
         self.assertEqual([layer1, layer3], self.timeline.get_layers())
         self.assertEqual([l.props.priority for l in [layer1, layer3]],
-                        list(range(2)))
+                         list(range(2)))
+
+    def test_group_ungroup_clips(self):
+        self.setup_timeline_container()
+
+        clip1 = common.create_test_clip(GES.TitleClip)
+        clip1.set_start(0 * Gst.SECOND)
+        clip1.set_duration(1 * Gst.SECOND)
+
+        uri = common.get_sample_uri("tears_of_steel.webm")
+        asset = GES.UriClipAsset.request_sync(uri)
+        clip2 = asset.extract()
+
+        self.layer.add_clip(clip1)
+        self.layer.add_clip(clip2)
+        # The selection does not care about GES.Groups, only about GES.Clips.
+        self.timeline_container.timeline.selection.select([clip1, clip2])
+
+        self.timeline_container.group_action.activate(None)
+        self.assertTrue(isinstance(clip1.get_parent(), GES.Group))
+        self.assertEqual(clip1.get_parent(), clip2.get_parent())
+
+        self.timeline_container.ungroup_action.activate(None)
+        self.assertIsNone(clip1.get_parent())
+        self.assertIsNone(clip2.get_parent())
+
+        for i in range(4):
+            # Undo ungrouping.
+            self.action_log.undo()
+            self.assertTrue(isinstance(clip1.get_parent(), GES.Group))
+            self.assertEqual(clip1.get_parent(), clip2.get_parent())
+
+            # Undo grouping.
+            self.action_log.undo()
+            self.assertIsNone(clip1.get_parent())
+            self.assertIsNone(clip2.get_parent())
+
+            # Redo grouping.
+            self.action_log.redo()
+            self.assertTrue(isinstance(clip1.get_parent(), GES.Group))
+            self.assertEqual(clip1.get_parent(), clip2.get_parent())
+
+            # Redo ungrouping.
+            self.action_log.redo()
+            self.assertIsNone(clip1.get_parent())
+            self.assertIsNone(clip2.get_parent())
 
 
 class TestLayerObserver(BaseTestUndoTimeline):
@@ -109,11 +162,11 @@ class TestLayerObserver(BaseTestUndoTimeline):
         layer3 = self.timeline.append_layer()
         self.assertEqual(self.timeline.get_layers(), [layer1, layer2, layer3])
 
-        timeline_ui = Timeline(container=None, app=self.app)
+        timeline_ui = Timeline(app=self.app, size_group=mock.Mock())
         timeline_ui.setProject(self.app.project_manager.current_project)
 
         # Click and drag a layer control box to move the layer.
-        with mock.patch.object(timeline_ui, 'get_event_widget') as get_event_widget:
+        with mock.patch.object(Gtk, 'get_event_widget') as get_event_widget:
             event = mock.Mock()
             event.get_button.return_value = True, 1
 
@@ -149,19 +202,14 @@ class TestLayerObserver(BaseTestUndoTimeline):
         self.action_log.redo()
         self.assertEqual(layer._nameIfSet(), "Beautiful name")
 
-    def testAddClip(self):
-        stacks = []
-        self.action_log.connect("commit", BaseTestUndoTimeline.commit_cb, stacks)
-
+    def test_add_clip(self):
         clip1 = GES.TitleClip()
         with self.action_log.started("add clip"):
             self.layer.add_clip(clip1)
 
-        self.assertEqual(1, len(stacks))
-        stack = stacks[0]
-        self.assertEqual(2, len(stack.done_actions), stack.done_actions)
+        stack, = self.action_log.undo_stacks
+        self.assertEqual(len(stack.done_actions), 1, stack.done_actions)
         self.assertTrue(isinstance(stack.done_actions[0], ClipAdded))
-        self.assertTrue(isinstance(stack.done_actions[1], AssetAddedAction))
         self.assertTrue(clip1 in self.getTimelineClips())
 
         self.action_log.undo()
@@ -192,7 +240,9 @@ class TestLayerObserver(BaseTestUndoTimeline):
         self.action_log.redo()
         self.assertFalse(clip1 in self.getTimelineClips())
 
-    def testUngroup(self):
+    def test_ungroup_group_clip(self):
+        # This test is in TestLayerObserver because the relevant operations
+        # recorded are clip-added and clip-removed.
         uri = common.get_sample_uri("tears_of_steel.webm")
         asset = GES.UriClipAsset.request_sync(uri)
         clip1 = asset.extract()
@@ -221,6 +271,14 @@ class TestLayerObserver(BaseTestUndoTimeline):
         self.assertEqual(5 * Gst.SECOND, timeline_clips[0].get_start())
         self.assertEqual(0.5 * Gst.SECOND, timeline_clips[0].get_duration())
 
+        self.action_log.redo()
+        timeline_clips = list(self.getTimelineClips())
+        self.assertEqual(2, len(timeline_clips), timeline_clips)
+        self.assertEqual(5 * Gst.SECOND, timeline_clips[0].get_start())
+        self.assertEqual(0.5 * Gst.SECOND, timeline_clips[0].get_duration())
+        self.assertEqual(5 * Gst.SECOND, timeline_clips[1].get_start())
+        self.assertEqual(0.5 * Gst.SECOND, timeline_clips[1].get_duration())
+
     def testSplitClip(self):
         clip = GES.TitleClip()
         clip.set_start(0 * Gst.SECOND)
@@ -358,8 +416,14 @@ class TestLayerObserver(BaseTestUndoTimeline):
         with self.action_log.started("add second clip"):
             self.layer.add_clip(clip2)
 
+        # Make sure the transition asset is ignored.
+        stack, = self.action_log.undo_stacks
+        for action in stack.done_actions:
+            self.assertNotIsInstance(action, AssetAddedAction,
+                                     stack.done_actions)
+
         def get_transition_element(ges_layer):
-            for clip in self.layer.get_clips():
+            for clip in ges_layer.get_clips():
                 if isinstance(clip, GES.TransitionClip):
                     for element in clip.get_children(False):
                         if isinstance(element, GES.VideoTransition):
@@ -519,7 +583,7 @@ class TestTimelineElementObserver(BaseTestUndoTimeline):
         self.action_log.redo()
         self.assertEqual(source.get_child_property("text")[1], "pigs fly!")
 
-    def testChangeEffectProperty(self):
+    def test_add_effect_change_property(self):
         stacks = []
         self.action_log.connect("commit", BaseTestUndoTimeline.commit_cb, stacks)
 
@@ -543,17 +607,33 @@ class TestTimelineElementObserver(BaseTestUndoTimeline):
 
         with self.action_log.started("change child property"):
             effect1.set_child_property("scratch-lines", 0)
-
         self.assertEqual(effect1.get_child_property("scratch-lines")[1], 0)
+
+        # Undo effect property change.
         self.action_log.undo()
         self.assertEqual(effect1.get_child_property("scratch-lines")[1], 7)
+
+        # Redo effect property change.
         self.action_log.redo()
         self.assertEqual(effect1.get_child_property("scratch-lines")[1], 0)
+
+        # Undo effect property change.
         self.action_log.undo()
         self.assertTrue(effect1 in clip1.get_children(True))
+
+        # Undo effect add.
         self.action_log.undo()
         self.assertFalse(effect1 in clip1.get_children(True))
 
+        # Redo effect add.
+        self.action_log.redo()
+        self.assertTrue(effect1 in clip1.get_children(True))
+        self.assertEqual(effect1.get_child_property("scratch-lines")[1], 7)
+
+        # Redo effect property change.
+        self.action_log.redo()
+        self.assertEqual(effect1.get_child_property("scratch-lines")[1], 0)
+
 
 class TestGObjectObserver(BaseTestUndoTimeline):
 
@@ -597,30 +677,32 @@ class TestGObjectObserver(BaseTestUndoTimeline):
 class TestDragDropUndo(BaseTestUndoTimeline):
 
     def test_clip_dragged_to_create_layer(self):
+        self.setup_timeline_container()
+        timeline_ui = self.timeline_container.timeline
         layers = self.timeline.get_layers()
         self.assertEqual(len(layers), 1)
 
         clip = GES.TitleClip()
         self.layer.add_clip(clip)
 
-        timeline_ui = Timeline(None, self.app)
-        timeline_ui.setProject(self.app.project_manager.current_project)
-        timeline_ui.get_parent = mock.MagicMock()
-
         # Drag a clip on a separator to create a layer.
-        timeline_ui.get_event_widget = mock.Mock(return_value=clip.ui)
-        event = mock.Mock()
-        event.x = 0
-        event.get_button.return_value = True, 1
-        timeline_ui._button_press_event_cb(None, event)
+        with mock.patch.object(Gtk, 'get_event_widget') as get_event_widget:
+            get_event_widget.return_value = clip.ui
 
-        def translate_coordinates(widget, x, y):
-            return x, y
-        clip.ui.translate_coordinates = translate_coordinates
-        event.get_state.return_value = Gdk.ModifierType.BUTTON1_MASK
-        event.x = 1
-        event.y = LAYER_HEIGHT * 2
-        timeline_ui._motion_notify_event_cb(None, event)
+            event = mock.Mock()
+            event.x = 0
+            event.get_button.return_value = True, 1
+            timeline_ui._button_press_event_cb(None, event)
+
+            def translate_coordinates(widget, x, y):
+                return x, y
+            clip.ui.translate_coordinates = translate_coordinates
+            event = mock.Mock()
+            event.get_state.return_value = Gdk.ModifierType.BUTTON1_MASK
+            event.x = 1
+            event.y = LAYER_HEIGHT * 2
+            event.get_button.return_value = True, 1
+            timeline_ui._motion_notify_event_cb(None, event)
 
         timeline_ui._button_release_event_cb(None, event)
 
@@ -645,20 +727,16 @@ class TestDragDropUndo(BaseTestUndoTimeline):
 
     def test_media_library_asset_dragged_on_separator(self):
         """Simulate dragging an asset from the media library to the timeline."""
+        self.setup_timeline_container()
+        timeline_ui = self.timeline_container.timeline
         project = self.app.project_manager.current_project
+        layers = self.timeline.get_layers()
+        self.assertEqual(len(layers), 1)
 
         uri = common.get_sample_uri("tears_of_steel.webm")
         asset = GES.UriClipAsset.request_sync(uri)
         self.assertTrue(project.add_asset(asset))
 
-        timeline_ui = Timeline(container=None, app=self.app)
-        timeline_ui.setProject(project)
-        timeline_ui.get_parent = mock.MagicMock()
-        timeline_ui.parent = mock.MagicMock()
-
-        layers = self.timeline.get_layers()
-        self.assertEqual(len(layers), 1)
-
         # Events emitted while dragging an asset on a separator in the timeline:
         # motion, receive, motion, leave, drop.
         with mock.patch.object(Gdk, "drag_status") as drag_status_mock:
@@ -687,7 +765,7 @@ class TestDragDropUndo(BaseTestUndoTimeline):
                 def translate_coordinates(widget, x, y):
                     return x, y
                 timeline_ui.translate_coordinates = translate_coordinates
-                timeline_ui._drag_motion_cb(None, None, CONTROL_WIDTH, LAYER_HEIGHT * 2, 0)
+                timeline_ui._drag_motion_cb(timeline_ui, None, 0, LAYER_HEIGHT * 2, 0)
                 self.assertFalse(timeline_ui.drag_get_data.called)
                 self.assertIsNotNone(timeline_ui.draggingElement)
                 self.assertTrue(timeline_ui.dropping_clips)
